@@ -2,63 +2,24 @@
 #define DOMDocumentImpl_HEADER_GUARD_
 
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2001-2002 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 2001, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Id: DOMDocumentImpl.hpp,v 1.14 2003/01/16 19:10:41 tng Exp $
+ * $Id: DOMDocumentImpl.hpp 568078 2007-08-21 11:43:25Z amassari $
  */
 
 //
@@ -70,14 +31,11 @@
 //  name is substituded for the *.
 //
 
-#include <string.h>
-#include <xercesc/util/XercesDefs.hpp>
 #include <xercesc/util/RefArrayOf.hpp>
-#include <xercesc/util/RefVectorOf.hpp>
 #include <xercesc/util/RefStackOf.hpp>
-#include <xercesc/util/RefHashTableOf.hpp>
+#include <xercesc/util/RefHash2KeysTableOf.hpp>
+#include <xercesc/util/StringPool.hpp>
 #include <xercesc/util/KeyRefPair.hpp>
-
 #include <xercesc/dom/DOMDocument.hpp>
 #include <xercesc/dom/DOMUserDataHandler.hpp>
 #include "DOMNodeImpl.hpp"
@@ -90,34 +48,36 @@ XERCES_CPP_NAMESPACE_BEGIN
 class DOMAttrImpl;
 class DOMCDATASectionImpl;
 class DOMCommentImpl;
+class DOMConfiguration;
 class DOMDeepNodeListImpl;
 class DOMDocumentFragmentImpl;
 class DOMDocumentTypeImpl;
 class DOMElementImpl;
 class DOMEntityImpl;
 class DOMEntityReferenceImpl;
-class DOMNodeImpl;
 class DOMNotationImpl;
 class DOMProcessingInstructionImpl;
 class DOMTextImpl;
 class DOMNodeIteratorImpl;
+class DOMNormalizer;
 class DOMTreeWalkerImpl;
 class DOMNodeFilter;
 class DOMNodeFilterImpl;
 class DOMImplementation;
 class DOMNodeIDMap;
 class DOMRangeImpl;
-class DOMParentNode;
 class DOMStringPool;
 class DOMBuffer;
+class MemoryManager;
+class XPathNSResolver;
+class XPathExpression;
 
 typedef RefVectorOf<DOMRangeImpl>        Ranges;
 typedef RefVectorOf<DOMNodeIteratorImpl>     NodeIterators;
 typedef KeyRefPair<void, DOMUserDataHandler> DOMUserDataRecord;
-typedef RefHashTableOf<DOMUserDataRecord> DOMNodeUserDataTable;
 typedef RefStackOf<DOMNode>               DOMNodePtr;
 
-class CDOM_EXPORT DOMDocumentImpl: public DOMDocument {
+class CDOM_EXPORT DOMDocumentImpl: public XMemory, public DOMDocument {
 public:
     // -----------------------------------------------------------------------
     //  data types
@@ -147,10 +107,11 @@ public:
     DOMNodeIDMap*         fNodeIDMap;     // for use by GetElementsById().
 
 public:
-    DOMDocumentImpl();
+    DOMDocumentImpl(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     DOMDocumentImpl(const XMLCh*     namespaceURI,     //DOM Level 2
                     const XMLCh*     qualifiedName,
-                    DOMDocumentType* doctype);
+                    DOMDocumentType* doctype,
+                    MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     virtual ~DOMDocumentImpl();
 
     void                         setDocumentType(DOMDocumentType *doctype);
@@ -195,6 +156,15 @@ public:
     virtual void                 removeRange(DOMRangeImpl* range); //non-standard api
     virtual void                 removeNodeIterator(DOMNodeIteratorImpl* nodeIterator); //non-standard api
 
+    virtual const DOMXPathExpression*    createExpression(const XMLCh *expression, const DOMXPathNSResolver *resolver);
+    virtual const DOMXPathNSResolver*    createNSResolver(DOMNode *nodeResolver);
+    virtual void* evaluate(const XMLCh *expression, DOMNode *contextNode, const DOMXPathNSResolver *resolver, 
+                           unsigned short type, void* result);
+
+
+    // Extension to be called by the Parser
+    DOMEntityReference*  createEntityReferenceByParser(const XMLCh * name);
+
 
     //
     // Functions to keep track of document mutations, so that node list chached
@@ -259,14 +229,10 @@ public:
     virtual void                 setDocumentURI(const XMLCh* documentURI);
     virtual bool                 getStrictErrorChecking() const;
     virtual void                 setStrictErrorChecking(bool strictErrorChecking);
-    virtual DOMErrorHandler*     getErrorHandler() const;
-    virtual void                 setErrorHandler(DOMErrorHandler* const handler);
     virtual DOMNode*             adoptNode(DOMNode* source);
     virtual void                 normalizeDocument();
-    virtual bool                 canSetNormalizationFeature(const XMLCh* const name, bool state) const;
-    virtual void                 setNormalizationFeature(const XMLCh* const name, bool state);
-    virtual bool                 getNormalizationFeature(const XMLCh* const name) const;
-
+    virtual DOMConfiguration*    getDOMConfiguration() const;
+    virtual void                 setDOMConfiguration(DOMConfiguration *config);
 
     // helper functions to prevent storing userdata pointers on every node.
     void*                        setUserData(DOMNodeImpl* n,
@@ -307,6 +273,7 @@ public:
     void                         releaseDocNotifyUserData(DOMNode* object);
     void                         releaseBuffer(DOMBuffer* buffer);
     DOMBuffer*                   popBuffer();
+    MemoryManager*               getMemoryManager() const;
 
     // Factory methods for getting/creating node lists.
     // Because nothing is ever deleted, the implementation caches and recycles
@@ -321,6 +288,12 @@ private:
     //Internal helper functions
     virtual DOMNode*             importNode(DOMNode *source, bool deep, bool cloningNode);
 
+    // -----------------------------------------------------------------------
+    // Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    DOMDocumentImpl(const DOMDocumentImpl &);
+    DOMDocumentImpl & operator = (const DOMDocumentImpl &);
+
 private:
     // -----------------------------------------------------------------------
     //  data
@@ -331,8 +304,10 @@ private:
     bool                  fStandalone;
     const XMLCh*          fVersion;
     const XMLCh*          fDocumentURI;
-
-    RefHashTableOf<DOMNodeUserDataTable>* fUserDataTable;
+    DOMConfiguration*     fDOMConfiguration;
+    
+    XMLStringPool         fUserDataTableKeys;
+    RefHash2KeysTableOf<DOMUserDataRecord>* fUserDataTable;
 
 
     // Per-Document heap Variables.
@@ -353,7 +328,8 @@ private:
     //
     void*                 fCurrentBlock;
     char*                 fFreePtr;
-    XMLSize_t             fFreeBytesRemaining;
+    XMLSize_t             fFreeBytesRemaining,
+                          fHeapAllocSize;
 
     // To recycle the DOMNode pointer
     RefArrayOf<DOMNodePtr>* fRecycleNodePtr;
@@ -368,15 +344,21 @@ private:
     DOMDocumentType*      fDocType;
     DOMElement*           fDocElement;
     DOMStringPool*        fNamePool;
-
+    DOMNormalizer*        fNormalizer;
     Ranges*               fRanges;
     NodeIterators*        fNodeIterators;
+    MemoryManager*        fMemoryManager;   // configurable memory manager
 
     int                   fChanges;
     bool                  errorChecking;    // Bypass error checking.
 
 };
 
+inline MemoryManager* DOMDocumentImpl::getMemoryManager() const
+{
+    return fMemoryManager;
+}
+
 XERCES_CPP_NAMESPACE_END
 
 // ---------------------------------------------------------------------------
@@ -405,11 +387,11 @@ inline void * operator new(size_t amt, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumen
 //    no matching operator delete found; memory will not be freed if initialization throws an exception
 // ---------------------------------------------------------------------------
 #if _MSC_VER >= 1200 /* VC++ 6.0 */
-inline void operator delete(void* ptr, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc)
+inline void operator delete(void* /*ptr*/, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * /*doc*/)
 {
     return;
 }
-inline void operator delete(void* ptr, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl::NodeObjectType type)
+inline void operator delete(void* /*ptr*/, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * /*doc*/, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl::NodeObjectType /*type*/)
 {
     return;
 }
