@@ -1,141 +1,32 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Log: XMLElementDecl.hpp,v $
- * Revision 1.4  2003/01/29 19:45:35  gareth
- * added api for DOMTypeInfo
- *
- * Revision 1.3  2002/11/04 15:00:21  tng
- * C++ Namespace Support.
- *
- * Revision 1.2  2002/08/20 16:55:33  tng
- * [Bug 6251] Info during compilation.
- *
- * Revision 1.1.1.1  2002/02/01 22:21:51  peiyongz
- * sane_include
- *
- * Revision 1.18  2001/08/21 16:06:10  tng
- * Schema: Unique Particle Attribution Constraint Checking.
- *
- * Revision 1.17  2001/07/24 18:30:47  knoaman
- * Added support for <group> + extra constraint checking for complexType
- *
- * Revision 1.16  2001/06/21 14:25:28  knoaman
- * Fix for bug 1946
- *
- * Revision 1.15  2001/05/11 13:25:32  tng
- * Copyright update.
- *
- * Revision 1.14  2001/05/03 20:34:22  tng
- * Schema: SchemaValidator update
- *
- * Revision 1.13  2001/04/19 18:16:52  tng
- * Schema: SchemaValidator update, and use QName in Content Model
- *
- * Revision 1.12  2001/03/21 21:56:02  tng
- * Schema: Add Schema Grammar, Schema Validator, and split the DTDValidator into DTDValidator, DTDScanner, and DTDGrammar.
- *
- * Revision 1.11  2001/03/21 19:29:29  tng
- * Schema: Content Model Updates, by Pei Yong Zhang.
- *
- * Revision 1.10  2001/02/26 19:29:13  tng
- * Schema: add virtual method getURI(), getContentSpec and setContenSpec in XMLElementDecl, and DTDElementDecl.
- *
- * Revision 1.9  2001/02/26 19:21:30  tng
- * Schema: add parameter prefix in findElem and findAttr.
- *
- * Revision 1.8  2000/12/14 18:49:57  tng
- * Fix API document generation warning: "Warning: end of member group without matching begin"
- *
- * Revision 1.7  2000/11/30 18:22:38  andyh
- * reuseValidator - fix bugs (spurious errors) that occured on reuse due to
- * pools already containing some items.  Fixed by Tinny Ng.
- *
- * Revision 1.6  2000/05/11 23:11:32  andyh
- * Add missing validity checks for stand-alone documents, character range
- * and Well-formed parsed entities.  Changes contributed by Sean MacRoibeaird
- * <sean.Macroibeaird@ireland.sun.com>
- *
- * Revision 1.5  2000/02/24 20:00:23  abagchi
- * Swat for removing Log from API docs
- *
- * Revision 1.4  2000/02/16 19:48:56  roddey
- * More documentation updates
- *
- * Revision 1.3  2000/02/15 01:21:30  roddey
- * Some initial documentation improvements. More to come...
- *
- * Revision 1.2  2000/02/06 07:47:48  rahulj
- * Year 2K copyright swat.
- *
- * Revision 1.1.1.1  1999/11/09 01:08:32  twl
- * Initial checkin
- *
- * Revision 1.2  1999/11/08 20:44:38  rahul
- * Swat for adding in Product name and CVS comment log variable.
- *
+ * $Id: XMLElementDecl.hpp 568078 2007-08-21 11:43:25Z amassari $
  */
 
 #if !defined(XMLELEMENTDECL_HPP)
 #define XMLELEMENTDECL_HPP
 
-#include <xercesc/util/XMLString.hpp>
 #include <xercesc/framework/XMLAttr.hpp>
 #include <xercesc/framework/XMLAttDefList.hpp>
+#include <xercesc/util/XMLString.hpp>
+#include <xercesc/util/PlatformUtils.hpp>
+#include <xercesc/internal/XSerializable.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -157,7 +48,7 @@ class XMLContentModel;
  *  with invalid or PCDATA element ids without having to know what type of
  *  validator its messing with.)
  */
-class XMLPARSER_EXPORT XMLElementDecl
+class XMLPARSER_EXPORT XMLElementDecl : public XSerializable, public XMemory
 {
  public:
     // -----------------------------------------------------------------------
@@ -185,6 +76,10 @@ class XMLPARSER_EXPORT XMLElementDecl
         , JustFaultIn
     };
 
+    /**
+     * @deprecated Use of addIfNotFound couldl produce undefined 
+     * behaviour in multithreaded environments.
+     */
     enum LookupOpts
     {
         AddIfNotFound
@@ -244,6 +139,8 @@ class XMLPARSER_EXPORT XMLElementDecl
       * defaulted in if not found. If it is defaulted in, then wasAdded should
       * be set, else it should be cleared. If its not found and the caller does
       * not want defaulting, then return a null pointer.
+      * Note that, in a multithreaded environment, it is dangerous for a 
+      * caller to invoke this method with options set to AddIfNotFound.
       *
       * @param  qName       This is the qName of the attribute, i.e. the actual
       *                     lexical name found.
@@ -454,11 +351,13 @@ class XMLPARSER_EXPORT XMLElementDecl
 
     /**
      * @return the uri part of DOM Level 3 TypeInfo
+     * @deprecated
      */
     virtual const XMLCh* getDOMTypeInfoUri() const = 0;
 
     /**
      * @return the name part of DOM Level 3 TypeInfo
+     * @deprecated
      */
     virtual const XMLCh* getDOMTypeInfoName() const = 0;
 
@@ -484,6 +383,15 @@ class XMLPARSER_EXPORT XMLElementDecl
 
     bool isExternal() const;
 
+    /** Get the memory manager
+      *
+      * This method returns the configurable memory manager used by the
+      * element declaration for dynamic allocation/deacllocation.
+      *
+      * @return the memory manager
+      */
+    MemoryManager* getMemoryManager() const;
+
     //@}
 
 
@@ -568,19 +476,37 @@ class XMLPARSER_EXPORT XMLElementDecl
 
     //@}
 
+    /***
+     * Support for Serialization/De-serialization
+     ***/
+    DECL_XSERIALIZABLE(XMLElementDecl)
+
+    enum objectType
+    {
+        Schema
+      , DTD
+      , UnKnown
+    };
+
+    virtual XMLElementDecl::objectType  getObjectType() const = 0;
+
+    static void            storeElementDecl(XSerializeEngine&        serEng
+                                          , XMLElementDecl*    const element);
+
+    static XMLElementDecl* loadElementDecl(XSerializeEngine& serEng);
 
 protected :
     // -----------------------------------------------------------------------
     //  Hidden constructors
     // -----------------------------------------------------------------------
-    XMLElementDecl();
+    XMLElementDecl(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
 
 private :
     // -----------------------------------------------------------------------
     //  Unimplemented constructors and operators
     // -----------------------------------------------------------------------
     XMLElementDecl(const XMLElementDecl&);
-    void operator=(const XMLElementDecl&);
+    XMLElementDecl& operator=(const XMLElementDecl&);
 
 
     // -----------------------------------------------------------------------
@@ -606,6 +532,7 @@ private :
     //  fExternalElement
     //      This flag indicates whether or the element was declared externally.
     // -----------------------------------------------------------------------
+    MemoryManager*      fMemoryManager;
     QName*              fElementName;
     CreateReasons       fCreateReason;
     unsigned int        fId;
@@ -667,6 +594,11 @@ inline bool XMLElementDecl::isExternal() const
     return fExternalElement;
 }
 
+inline MemoryManager* XMLElementDecl::getMemoryManager() const
+{
+    return fMemoryManager;
+}
+
 
 // ---------------------------------------------------------------------------
 //  XMLElementDecl: Setter methods
