@@ -1,118 +1,28 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Log: XMLFormatter.hpp,v $
- * Revision 1.7  2003/01/31 00:30:48  jberry
- * Syntax error in declaration
- *
- * Revision 1.6  2003/01/28 18:32:33  peiyongz
- * Bug#13694: Allow Xerces to write the BOM to XML files
- *
- * Revision 1.5  2003/01/24 20:20:22  tng
- * Add method flush to XMLFormatTarget
- *
- * Revision 1.4  2002/11/04 15:00:21  tng
- * C++ Namespace Support.
- *
- * Revision 1.3  2002/07/30 16:29:16  tng
- * [Bug 8550] No explanation of XMLFormatter escape options.
- *
- * Revision 1.2  2002/06/21 19:31:23  peiyongz
- * getTranscoder() added;
- *
- * Revision 1.1.1.1  2002/02/01 22:21:52  peiyongz
- * sane_include
- *
- * Revision 1.7  2000/10/17 19:25:38  andyh
- * XMLFormatTarget, removed version of writeChars with no length.  Can not be
- * safely used, and obscured other errors.
- *
- * Revision 1.6  2000/10/10 23:54:58  andyh
- * XMLFormatter patch, contributed by Bill Schindler.  Fix problems with
- * output to multi-byte encodings.
- *
- * Revision 1.5  2000/04/07 01:01:56  roddey
- * Fixed an error message so that it indicated the correct radix for the rep
- * token. Get all of the basic output formatting functionality in place for
- * at least ICU and Win32 transcoders.
- *
- * Revision 1.4  2000/04/06 23:50:38  roddey
- * Now the low level formatter handles doing char refs for
- * unrepresentable chars (in addition to the replacement char style
- * already done.)
- *
- * Revision 1.3  2000/04/06 19:09:21  roddey
- * Some more improvements to output formatting. Now it will correctly
- * handle doing the 'replacement char' style of dealing with chars
- * that are unrepresentable.
- *
- * Revision 1.2  2000/04/05 00:20:16  roddey
- * More updates for the low level formatted output support
- *
- * Revision 1.1  2000/03/28 19:43:17  roddey
- * Fixes for signed/unsigned warnings. New work for two way transcoding
- * stuff.
- *
+ * $Id: XMLFormatter.hpp 568078 2007-08-21 11:43:25Z amassari $
  */
 
 #if !defined(XMLFORMATTER_HPP)
 #define XMLFORMATTER_HPP
 
-#include <xercesc/util/XercesDefs.hpp>
+#include <xercesc/util/PlatformUtils.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -128,7 +38,7 @@ class XMLTranscoder;
  *  A number of flags are provided to control whether various optional
  *  formatting operations are performed.
  */
-class XMLPARSER_EXPORT XMLFormatter
+class XMLPARSER_EXPORT XMLFormatter : public XMemory
 {
 public:
     // -----------------------------------------------------------------------
@@ -233,7 +143,7 @@ public:
      * Fail the operation.</p>
      * <p><code>UnRep_CharRef:</code>
      * Display the unrepresented character as reference.</p>
-     * <p><code>UnRep_CharRef:</code>
+     * <p><code>UnRep_Replace:</code>
      * Replace the unrepresented character with the replacement character.</p>
      * <p><code>DefaultUnRep:</code>
      * Special value, do not use directly.</p>
@@ -257,16 +167,40 @@ public:
     //@{
     /**
      * @param outEncoding the encoding for the formatted content
+     * @param docVersion  
      * @param target      the formatTarget where the formatted content is written to
      * @param escapeFlags the escape style for certain character
      * @param unrepFlags  the reaction to unrepresentable character
+     * @param manager     Pointer to the memory manager to be used to
+     *                    allocate objects.
      */
+    XMLFormatter
+    (
+        const   XMLCh* const            outEncoding
+        , const XMLCh* const            docVersion
+        ,       XMLFormatTarget* const  target
+        , const EscapeFlags             escapeFlags = NoEscapes
+        , const UnRepFlags              unrepFlags = UnRep_Fail
+        ,       MemoryManager* const    manager = XMLPlatformUtils::fgMemoryManager
+    );
+
+    XMLFormatter
+    (
+        const   char* const             outEncoding
+        , const char* const             docVersion
+        ,       XMLFormatTarget* const  target
+        , const EscapeFlags             escapeFlags = NoEscapes
+        , const UnRepFlags              unrepFlags = UnRep_Fail
+        ,       MemoryManager* const    manager = XMLPlatformUtils::fgMemoryManager
+    );
+
     XMLFormatter
     (
         const   XMLCh* const            outEncoding
         ,       XMLFormatTarget* const  target
         , const EscapeFlags             escapeFlags = NoEscapes
         , const UnRepFlags              unrepFlags = UnRep_Fail
+        ,       MemoryManager* const    manager = XMLPlatformUtils::fgMemoryManager
     );
 
     XMLFormatter
@@ -275,8 +209,9 @@ public:
         ,       XMLFormatTarget* const  target
         , const EscapeFlags             escapeFlags = NoEscapes
         , const UnRepFlags              unrepFlags = UnRep_Fail
+        ,       MemoryManager* const    manager = XMLPlatformUtils::fgMemoryManager
     );
-
+    
     ~XMLFormatter();
     //@}
 
@@ -334,7 +269,8 @@ public:
      * @return return the transcoder used internally for transcoding the formatter conent
      */
     inline const XMLTranscoder*   getTranscoder() const;
-    //@}
+
+   //@}
 
     // -----------------------------------------------------------------------
     //  Setter methods
@@ -377,13 +313,29 @@ public:
     //@}
 
 
+    // -----------------------------------------------------------------------
+    //  Getter methods
+    // -----------------------------------------------------------------------
+    /** @name Setter methods */
+    //@{
+    /**
+     * @return return the escape style for the formatted content
+     */
+    EscapeFlags getEscapeFlags() const;
+
+    /**
+     * @return return the reaction for unrepresentable character
+     */
+    UnRepFlags getUnRepFlags() const;
+    //@}
+
 private :
     // -----------------------------------------------------------------------
     //  Unimplemented constructors and operators
     // -----------------------------------------------------------------------
     XMLFormatter();
     XMLFormatter(const XMLFormatter&);
-    void operator=(const XMLFormatter&);
+    XMLFormatter& operator=(const XMLFormatter&);
 
 
     // -----------------------------------------------------------------------
@@ -398,11 +350,20 @@ private :
     // -----------------------------------------------------------------------
     //  Private helper methods
     // -----------------------------------------------------------------------
-    const XMLByte* getAposRef(unsigned int & count);
-    const XMLByte* getAmpRef(unsigned int & count);
-    const XMLByte* getGTRef(unsigned int & count);
-    const XMLByte* getLTRef(unsigned int & count);
-    const XMLByte* getQuoteRef(unsigned int & count);
+    const XMLByte* getCharRef(unsigned int & count, 
+                              XMLByte*      &ref, 
+                              const XMLCh *  stdRef);  
+ 
+    void writeCharRef(const XMLCh &toWrite);
+    void writeCharRef(unsigned long toWrite);
+
+    bool inEscapeList(const XMLFormatter::EscapeFlags escStyle
+                    , const XMLCh                     toCheck);
+                              
+
+    unsigned int handleUnEscapedChars(const XMLCh *                  srcPtr, 
+                                      const unsigned int             count, 
+                                      const UnRepFlags               unrepFlags);
 
     void specialFormat
     (
@@ -447,6 +408,11 @@ private :
     //      These are character refs for the standard char refs, in the
     //      output encoding. They are faulted in as required, by transcoding
     //      them from fixed Unicode versions.
+    //
+    //  fIsXML11
+    //      for performance reason, we do not store the actual version string
+    //      and do the string comparison again and again.
+    //
     // -----------------------------------------------------------------------
     EscapeFlags                 fEscapeFlags;
     XMLCh*                      fOutEncoding;
@@ -454,7 +420,6 @@ private :
     UnRepFlags                  fUnRepFlags;
     XMLTranscoder*              fXCoder;
     XMLByte                     fTmpBuf[kTmpBufSize + 4];
-
     XMLByte*                    fAposRef;
     unsigned int                fAposLen;
     XMLByte*                    fAmpRef;
@@ -465,10 +430,12 @@ private :
     unsigned int                fLTLen;
     XMLByte*                    fQuoteRef;
     unsigned int                fQuoteLen;
+    bool                        fIsXML11;
+    MemoryManager*              fMemoryManager;
 };
 
 
-class XMLPARSER_EXPORT XMLFormatTarget
+class XMLPARSER_EXPORT XMLFormatTarget : public XMemory
 {
 public:
     // -----------------------------------------------------------------------
@@ -482,7 +449,7 @@ public:
     // -----------------------------------------------------------------------
     virtual void writeChars
     (
-        const   XMLByte* const      toWrite
+          const XMLByte* const      toWrite
         , const unsigned int        count
         ,       XMLFormatter* const formatter
     ) = 0;
@@ -494,9 +461,14 @@ protected :
     // -----------------------------------------------------------------------
     //  Hidden constructors and operators
     // -----------------------------------------------------------------------
-    XMLFormatTarget() {}
-    XMLFormatTarget(const XMLFormatTarget&) {}
-    void operator=(const XMLFormatTarget&) {}
+    XMLFormatTarget() {};
+
+private:
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    XMLFormatTarget(const XMLFormatTarget&);
+    XMLFormatTarget& operator=(const XMLFormatTarget&);
 };
 
 
@@ -539,6 +511,19 @@ inline XMLFormatter& XMLFormatter::operator<<(const UnRepFlags newFlags)
     return *this;
 }
 
+// ---------------------------------------------------------------------------
+//  XMLFormatter: Getter methods
+// ---------------------------------------------------------------------------
+inline XMLFormatter::EscapeFlags XMLFormatter::getEscapeFlags() const
+{
+    return fEscapeFlags;
+}
+
+inline XMLFormatter::UnRepFlags XMLFormatter::getUnRepFlags() const
+{
+    return fUnRepFlags;
+}
+
 XERCES_CPP_NAMESPACE_END
 
 #endif
