@@ -1,123 +1,25 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Log: BinHTTPURLInputStream.cpp,v $
- * Revision 1.3  2002/12/06 16:43:55  tng
- * Fix the error messages thrown from net accessor module.
- *
- * Revision 1.2  2002/11/04 15:11:39  tng
- * C++ Namespace Support.
- *
- * Revision 1.1.1.1  2002/02/01 22:22:23  peiyongz
- * sane_include
- *
- * Revision 1.12  2001/12/13 21:16:52  peiyongz
- * Fix: Invalid Argument to FreeLibrary (Hint: 0x0000000)
- *
- * Revision 1.11  2001/10/25 16:26:39  tng
- * [Bug 4213] BinHTTPURLInputStream initialisation not thread safe.  By Mark Weaver.
- *
- * Revision 1.10  2001/10/25 16:10:46  tng
- * [Bug 4121] BinHTTPUrlInputStream needds to read entire HTTP header. By John Clayton.
- *
- * Revision 1.9  2001/10/24 20:17:54  tng
- * [Bug 3813] BinHTTPURLInputStream has weak HTTP request capabilities.  By Kevin Philips.
- *
- * Revision 1.8  2001/10/24 20:03:03  tng
- * [Bug 2305] Include <stdlib.h> to BinHTTPURLInputStream.cpp.  By Peter A. Volchek.
- *
- * Revision 1.7  2001/09/04 17:52:57  peiyongz
- * Bugzilla# 3170: patch from Kevin Philips to handle Query in XMLURL.
- *
- * Revision 1.6  2001/01/22 16:43:38  tng
- * Loads winsock dynamically.  Fixed by Curt Arnold.
- * Winsock2 is not initialized unless an http URL is used.    If an http
- * URL is used and the Winsock 2 DLL is not installed, then an NetAccessor
- * initialization exception is thrown.
- *
- * Revision 1.5  2000/07/21 03:22:44  andyh
- * Improved (but still weak) http access by the parser.
- * Windows only.  UNIX will follow, probably tomorrow.
- *
- * Revision 1.4  2000/05/15 22:31:29  andyh
- * Replace #include<memory.h> with <string.h> everywhere.
- *
- * Revision 1.3  2000/03/24 00:32:15  rahulj
- * Connect to the port specified in the URL, rather than the default one.
- *
- * Revision 1.2  2000/03/22 00:21:10  rahulj
- * Now we throw exceptions when errors occur.
- * Simplified the code, based on the assumption that
- * the calling function will make sure that the buffer into
- * which the data has to be read is large enough.
- *
- * Revision 1.1  2000/03/17 02:37:54  rahulj
- * First cut at adding HTTP capability via native sockets.
- * Still need to add:
- *   error handling capability, ports other than 80,
- *   escaped URL's
- * Will add options in project file only when I am done with these
- * above changes.
- *
+ * $Id: BinHTTPURLInputStream.cpp 568078 2007-08-21 11:43:25Z amassari $
  */
 
 
-#define _WINSOCKAPI_
-
 #define INCL_WINSOCK_API_TYPEDEFS 1
 #include <winsock2.h>
 #include <windows.h>
@@ -127,7 +29,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-
 #include <xercesc/util/PlatformUtils.hpp>
 #include <xercesc/util/XMLNetAccessor.hpp>
 #include <xercesc/util/NetAccessors/WinSock/BinHTTPURLInputStream.hpp>
@@ -135,26 +36,60 @@
 #include <xercesc/util/XMLExceptMsgs.hpp>
 #include <xercesc/util/Janitor.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
+#include <xercesc/util/Base64.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
-HMODULE gWinsockLib = NULL;
-LPFN_GETHOSTBYNAME gWSgethostbyname = NULL;
-LPFN_INET_ADDR gWSinet_addr = NULL;
-LPFN_GETHOSTBYADDR gWSgethostbyaddr = NULL;
-LPFN_HTONS gWShtons = NULL;
-LPFN_SOCKET gWSsocket = NULL;
-LPFN_CONNECT gWSconnect = NULL;
-LPFN_SEND gWSsend = NULL;
-LPFN_RECV gWSrecv = NULL;
-LPFN_SHUTDOWN gWSshutdown = NULL;
-LPFN_CLOSESOCKET gWSclosesocket = NULL;
-LPFN_WSACLEANUP gWSACleanup = NULL;
+class SocketJanitor
+{
+public:
+    // -----------------------------------------------------------------------
+    //  Constructors and Destructor
+    // -----------------------------------------------------------------------
+    SocketJanitor(SOCKET* toDelete) : fData(toDelete) {}
+    ~SocketJanitor() { reset(); }
+
+    SOCKET* get() const { return fData; }
+    SOCKET* release() {	SOCKET* p = fData; fData = 0; return p; }
+
+    void reset(SOCKET* p = 0) { if(fData) BinHTTPURLInputStream::closesocket(*fData); fData=p; }
+    bool isDataNull() { return (fData == 0); }
+
+private :
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    SocketJanitor();
+    SocketJanitor(const SocketJanitor&);
+    SocketJanitor& operator=(const SocketJanitor&);
+
+    // -----------------------------------------------------------------------
+    //  Private data members
+    //
+    //  fData
+    //      This is the pointer to the socket that must be closed when 
+    //      this object is destroyed.
+    // -----------------------------------------------------------------------
+    SOCKET*  fData;
+};
+
+static HMODULE gWinsockLib = NULL;
+static LPFN_GETHOSTBYNAME gWSgethostbyname = NULL;
+static LPFN_INET_ADDR gWSinet_addr = NULL;
+static LPFN_GETHOSTBYADDR gWSgethostbyaddr = NULL;
+static LPFN_HTONS gWShtons = NULL;
+static LPFN_SOCKET gWSsocket = NULL;
+static LPFN_CONNECT gWSconnect = NULL;
+static LPFN_SEND gWSsend = NULL;
+static LPFN_RECV gWSrecv = NULL;
+static LPFN_SHUTDOWN gWSshutdown = NULL;
+static LPFN_CLOSESOCKET gWSclosesocket = NULL;
+static LPFN_WSACLEANUP gWSACleanup = NULL;
 
 bool BinHTTPURLInputStream::fInitialized = false;
 XMLMutex* BinHTTPURLInputStream::fInitMutex = 0;
 
-void BinHTTPURLInputStream::Initialize() {
+void BinHTTPURLInputStream::Initialize(MemoryManager* const manager) {
     //
     // Initialize the WinSock library here.
     //
@@ -165,21 +100,21 @@ void BinHTTPURLInputStream::Initialize() {
 	if(gWinsockLib == NULL) {
 		gWinsockLib = LoadLibrary(_T("WSOCK32"));
 		if(gWinsockLib == NULL) {
-			ThrowXML(NetAccessorException, XMLExcepts::NetAcc_InitFailed);
+			ThrowXMLwithMemMgr(NetAccessorException, XMLExcepts::NetAcc_InitFailed, manager);
 		}
 		else {
-			startup = (LPFN_WSASTARTUP) GetProcAddress(gWinsockLib,_T("WSAStartup"));
-			gWSACleanup = (LPFN_WSACLEANUP) GetProcAddress(gWinsockLib,_T("WSACleanup"));
-			gWSgethostbyname = (LPFN_GETHOSTBYNAME) GetProcAddress(gWinsockLib,_T("gethostbyname"));
-			gWSinet_addr = (LPFN_INET_ADDR) GetProcAddress(gWinsockLib,_T("inet_addr"));
-			gWSgethostbyaddr = (LPFN_GETHOSTBYADDR) GetProcAddress(gWinsockLib,_T("gethostbyaddr"));
-			gWShtons = (LPFN_HTONS) GetProcAddress(gWinsockLib,_T("htons"));
-			gWSsocket = (LPFN_SOCKET) GetProcAddress(gWinsockLib,_T("socket"));
-			gWSconnect = (LPFN_CONNECT) GetProcAddress(gWinsockLib,_T("connect"));
-			gWSsend = (LPFN_SEND) GetProcAddress(gWinsockLib,_T("send"));
-			gWSrecv = (LPFN_RECV) GetProcAddress(gWinsockLib,_T("recv"));
-			gWSshutdown = (LPFN_SHUTDOWN) GetProcAddress(gWinsockLib,_T("shutdown"));
-			gWSclosesocket = (LPFN_CLOSESOCKET) GetProcAddress(gWinsockLib,_T("closesocket"));
+			startup = (LPFN_WSASTARTUP) GetProcAddress(gWinsockLib,"WSAStartup");
+			gWSACleanup = (LPFN_WSACLEANUP) GetProcAddress(gWinsockLib,"WSACleanup");
+			gWSgethostbyname = (LPFN_GETHOSTBYNAME) GetProcAddress(gWinsockLib,"gethostbyname");
+			gWSinet_addr = (LPFN_INET_ADDR) GetProcAddress(gWinsockLib,"inet_addr");
+			gWSgethostbyaddr = (LPFN_GETHOSTBYADDR) GetProcAddress(gWinsockLib,"gethostbyaddr");
+			gWShtons = (LPFN_HTONS) GetProcAddress(gWinsockLib,"htons");
+			gWSsocket = (LPFN_SOCKET) GetProcAddress(gWinsockLib,"socket");
+			gWSconnect = (LPFN_CONNECT) GetProcAddress(gWinsockLib,"connect");
+			gWSsend = (LPFN_SEND) GetProcAddress(gWinsockLib,"send");
+			gWSrecv = (LPFN_RECV) GetProcAddress(gWinsockLib,"recv");
+			gWSshutdown = (LPFN_SHUTDOWN) GetProcAddress(gWinsockLib,"shutdown");
+			gWSclosesocket = (LPFN_CLOSESOCKET) GetProcAddress(gWinsockLib,"closesocket");
 
 			if(startup == NULL ||
 				gWSACleanup == NULL ||
@@ -196,7 +131,7 @@ void BinHTTPURLInputStream::Initialize() {
 			{
 				gWSACleanup = NULL;
 				Cleanup();
-				ThrowXML(NetAccessorException, XMLExcepts::NetAcc_InitFailed);
+				ThrowXMLwithMemMgr(NetAccessorException, XMLExcepts::NetAcc_InitFailed, manager);
 			}
 		}
 	}
@@ -205,7 +140,7 @@ void BinHTTPURLInputStream::Initialize() {
     if (err != 0)
     {
         // Call WSAGetLastError() to get the last error.
-        ThrowXML(NetAccessorException, XMLExcepts::NetAcc_InitFailed);
+        ThrowXMLwithMemMgr(NetAccessorException, XMLExcepts::NetAcc_InitFailed, manager);
     }
     fInitialized = true;
 }
@@ -228,9 +163,9 @@ void BinHTTPURLInputStream::Cleanup() {
 		gWSshutdown = NULL;
 		gWSclosesocket = NULL;
 
-      fInitialized = false;
-      delete fInitMutex;
-      fInitMutex = 0;
+        fInitialized = false;
+        delete fInitMutex;
+        fInitMutex = 0;
 	}
 }
 
@@ -255,22 +190,22 @@ unsigned short BinHTTPURLInputStream::htons(unsigned short hostshort)
 	return (*gWShtons)(hostshort);
 }
 
-unsigned short BinHTTPURLInputStream::socket(int af,int type,int protocol)
+unsigned int BinHTTPURLInputStream::socket(int af,int type,int protocol)
 {
 	return (*gWSsocket)(af,type,protocol);
 }
 
-int BinHTTPURLInputStream::connect(unsigned short s,const sockaddr* name,int namelen)
+int BinHTTPURLInputStream::connect(unsigned int s,const sockaddr* name,int namelen)
 {
 	return (*gWSconnect)(s,name,namelen);
 }
 
-int BinHTTPURLInputStream::send(unsigned short s,const char* buf,int len,int flags)
+int BinHTTPURLInputStream::send(unsigned int s,const char* buf,int len,int flags)
 {
 	return (*gWSsend)(s,buf,len,flags);
 }
 
-int BinHTTPURLInputStream::recv(unsigned short s,char* buf,int len,int flags)
+int BinHTTPURLInputStream::recv(unsigned int s,char* buf,int len,int flags)
 {
 	return (*gWSrecv)(s,buf,len,flags);
 }
@@ -286,7 +221,7 @@ int BinHTTPURLInputStream::closesocket(unsigned int socket)
 }
 
 
-BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
+BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource, const XMLNetHTTPInfo* httpInfo /*=0*/)
       : fSocketHandle(0)
       , fBytesProcessed(0)
 {
@@ -294,7 +229,7 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     {
         if (!fInitMutex)
         {
-            XMLMutex* tmpMutex = new XMLMutex;
+            XMLMutex* tmpMutex = new XMLMutex(XMLPlatformUtils::fgMemoryManager);
             if (XMLPlatformUtils::compareAndSwap((void**)&fInitMutex, tmpMutex, 0))
             {
                 // Someone beat us to it, so let's clean up ours
@@ -304,33 +239,34 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
          XMLMutexLock lock(fInitMutex);
          if (!fInitialized)
          {
-             Initialize();
+             Initialize(urlSource.getMemoryManager());
          }
     }
 
+    fMemoryManager = urlSource.getMemoryManager();
     //
     // Pull all of the parts of the URL out of th urlSource object, and transcode them
     //   and transcode them back to ASCII.
     //
     const XMLCh*        hostName = urlSource.getHost();
-    char*               hostNameAsCharStar = XMLString::transcode(hostName);
-    ArrayJanitor<char>  janBuf1(hostNameAsCharStar);
+    char*               hostNameAsCharStar = XMLString::transcode(hostName, urlSource.getMemoryManager());
+    ArrayJanitor<char>  janBuf1(hostNameAsCharStar, urlSource.getMemoryManager());
 
     const XMLCh*        path = urlSource.getPath();
-    char*               pathAsCharStar = XMLString::transcode(path);
-    ArrayJanitor<char>  janBuf2(pathAsCharStar);
+    char*               pathAsCharStar = XMLString::transcode(path, urlSource.getMemoryManager());
+    ArrayJanitor<char>  janBuf2(pathAsCharStar, urlSource.getMemoryManager());
 
     const XMLCh*        fragment = urlSource.getFragment();
     char*               fragmentAsCharStar = 0;
     if (fragment)
-        fragmentAsCharStar = XMLString::transcode(fragment);
-    ArrayJanitor<char>  janBuf3(fragmentAsCharStar);
+        fragmentAsCharStar = XMLString::transcode(fragment, urlSource.getMemoryManager());
+    ArrayJanitor<char>  janBuf3(fragmentAsCharStar, urlSource.getMemoryManager());
 
     const XMLCh*        query = urlSource.getQuery();
     char*               queryAsCharStar = 0;
     if (query)
-        queryAsCharStar = XMLString::transcode(query);
-    ArrayJanitor<char>  janBuf4(queryAsCharStar);		
+        queryAsCharStar = XMLString::transcode(query, urlSource.getMemoryManager());
+    ArrayJanitor<char>  janBuf4(queryAsCharStar, urlSource.getMemoryManager());		
 
     unsigned short      portNumber = (unsigned short) urlSource.getPortNum();
 
@@ -347,16 +283,16 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
         if (numAddress == INADDR_NONE)
         {
             // Call WSAGetLastError() to get the error number.
-            ThrowXML1(NetAccessorException,
-                     XMLExcepts::NetAcc_TargetResolution, hostName);
+            ThrowXMLwithMemMgr1(NetAccessorException,
+                     XMLExcepts::NetAcc_TargetResolution, hostName, fMemoryManager);
         }
         if ((hostEntPtr =
                 gethostbyaddr((const char *) &numAddress,
                               sizeof(unsigned long), AF_INET)) == NULL)
         {
             // Call WSAGetLastError() to get the error number.
-            ThrowXML1(NetAccessorException,
-                     XMLExcepts::NetAcc_TargetResolution, hostName);
+            ThrowXMLwithMemMgr1(NetAccessorException,
+                     XMLExcepts::NetAcc_TargetResolution, hostName, fMemoryManager);
         }
     }
 
@@ -369,15 +305,16 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     if (s == INVALID_SOCKET)
     {
         // Call WSAGetLastError() to get the error number.
-        ThrowXML1(NetAccessorException,
-                 XMLExcepts::NetAcc_CreateSocket, urlSource.getURLText());
+        ThrowXMLwithMemMgr1(NetAccessorException,
+                 XMLExcepts::NetAcc_CreateSocket, urlSource.getURLText(), fMemoryManager);
     }
+    SocketJanitor janSock(&s);
 
     if (connect(s, (struct sockaddr *) &sa, sizeof(sa)) == SOCKET_ERROR)
     {
         // Call WSAGetLastError() to get the error number.
-        ThrowXML1(NetAccessorException,
-                 XMLExcepts::NetAcc_ConnSocket, urlSource.getURLText());
+        ThrowXMLwithMemMgr1(NetAccessorException,
+                 XMLExcepts::NetAcc_ConnSocket, urlSource.getURLText(), fMemoryManager);
     }
 
 
@@ -391,7 +328,15 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
 
     memset(fBuffer, 0, sizeof(fBuffer));
 
-    strcpy(fBuffer, "GET ");
+    if(httpInfo==0)
+        strcpy(fBuffer, "GET ");
+    else {
+        switch(httpInfo->fHTTPMethod) {
+        case XMLNetHTTPInfo::GET:   strcpy(fBuffer, "GET "); break;
+        case XMLNetHTTPInfo::PUT:   strcpy(fBuffer, "PUT "); break;
+        case XMLNetHTTPInfo::POST:  strcpy(fBuffer, "POST "); break;
+        }
+    }
     strcat(fBuffer, pathAsCharStar);
 
     if (queryAsCharStar != 0)
@@ -416,7 +361,43 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
         int i = strlen(fBuffer);
         _itoa(portNumber, fBuffer+i, 10);
     }
-    strcat(fBuffer, "\r\n\r\n");
+    strcat(fBuffer, "\r\n");
+
+    const XMLCh* username = urlSource.getUser();
+    const XMLCh* password = urlSource.getPassword();
+    if (username && password)
+    {
+        XMLBuffer userPass(256, fMemoryManager);
+        userPass.append(username);
+        userPass.append(chColon);
+        userPass.append(password);
+        char* userPassAsCharStar = XMLString::transcode(userPass.getRawBuffer(), fMemoryManager);
+        ArrayJanitor<char>  janBuf(userPassAsCharStar, fMemoryManager);
+
+        unsigned int len;
+        XMLByte* encodedData = Base64::encode((XMLByte *)userPassAsCharStar, strlen(userPassAsCharStar), &len, fMemoryManager);
+        ArrayJanitor<XMLByte>  janBuf2(encodedData, fMemoryManager);
+ 
+        if (encodedData)
+        {
+            // HTTP doesn't want the 0x0A separating the data in chunks of 76 chars per line
+            XMLByte* authData = (XMLByte*)fMemoryManager->allocate((len+1)*sizeof(XMLByte));
+            ArrayJanitor<XMLByte>  janBuf(authData, fMemoryManager);
+            XMLByte* cursor=authData;
+            for(unsigned int i=0;i<len;i++)
+                if(encodedData[i]!=chLF)
+                    *cursor++=encodedData[i];
+            *cursor++=0;
+            strcat(fBuffer, "Authorization: Basic ");
+            strcat(fBuffer, (char*)authData);
+            strcat(fBuffer, "\r\n");
+        }
+    }
+
+    if(httpInfo!=0 && httpInfo->fHeaders!=0)
+        strncat(fBuffer,httpInfo->fHeaders,httpInfo->fHeadersLen);
+
+    strcat(fBuffer, "\r\n");
 
     // Send the http request
     int lent = strlen(fBuffer);
@@ -424,10 +405,19 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     if ((aLent = send(s, fBuffer, lent, 0)) != lent)
     {
         // Call WSAGetLastError() to get the error number.
-        ThrowXML1(NetAccessorException,
-                 XMLExcepts::NetAcc_WriteSocket, urlSource.getURLText());
+        ThrowXMLwithMemMgr1(NetAccessorException,
+                 XMLExcepts::NetAcc_WriteSocket, urlSource.getURLText(), fMemoryManager);
     }
 
+    if(httpInfo!=0 && httpInfo->fPayload!=0) {
+        int  aLent = 0;
+        if ((aLent = send(s, httpInfo->fPayload, httpInfo->fPayloadLen, 0)) != httpInfo->fPayloadLen)
+        {
+            // Call WSAGetLastError() to get the error number.
+            ThrowXMLwithMemMgr1(NetAccessorException,
+                     XMLExcepts::NetAcc_WriteSocket, urlSource.getURLText(), fMemoryManager);
+        }
+    }
 
     //
     // get the response, check the http header for errors from the server.
@@ -437,7 +427,7 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     if (aLent == SOCKET_ERROR || aLent == 0)
     {
         // Call WSAGetLastError() to get the error number.
-        ThrowXML1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText());
+        ThrowXMLwithMemMgr1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText(), fMemoryManager);
     }
 
     fBufferEnd = fBuffer+aLent;
@@ -472,7 +462,7 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
                 if (aLent == SOCKET_ERROR || aLent == 0)
                 {
                     // Call WSAGetLastError() to get the error number.
-                    ThrowXML1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText());
+                    ThrowXMLwithMemMgr1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText(), fMemoryManager);
                 }
                 fBufferEnd = fBufferEnd + aLent;
                 *fBufferEnd = 0;
@@ -485,13 +475,13 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     char *p = strstr(fBuffer, "HTTP");
     if (p == 0)
     {
-        ThrowXML1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText());
+        ThrowXMLwithMemMgr1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText(), fMemoryManager);
     }
 
     p = strchr(p, ' ');
     if (p == 0)
     {
-        ThrowXML1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText());
+        ThrowXMLwithMemMgr1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText(), fMemoryManager);
     }
 
     int httpResponse = atoi(p);
@@ -500,10 +490,10 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
         // Most likely a 404 Not Found error.
         //   Should recognize and handle the forwarding responses.
         //
-        ThrowXML1(NetAccessorException, XMLExcepts::File_CouldNotOpenFile, urlSource.getURLText());
+        ThrowXMLwithMemMgr1(NetAccessorException, XMLExcepts::File_CouldNotOpenFile, urlSource.getURLText(), fMemoryManager);
     }
 
-    fSocketHandle = (unsigned int) s;
+    fSocketHandle = (unsigned int) *janSock.release();
 }
 
 
@@ -540,7 +530,7 @@ unsigned int BinHTTPURLInputStream::readBytes(XMLByte* const    toFill
         if (len == SOCKET_ERROR)
         {
             // Call WSAGetLastError() to get the error number.
-            ThrowXML(NetAccessorException, XMLExcepts::NetAcc_ReadSocket);
+            ThrowXMLwithMemMgr(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, fMemoryManager);
         }
     }
 
