@@ -1,61 +1,22 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Id: MacOSPlatformUtils.cpp,v 1.19 2003/07/31 18:37:29 jberry Exp $
+ * $Id: MacOSPlatformUtils.cpp 568078 2007-08-21 11:43:25Z amassari $
  */
 
 
@@ -68,6 +29,7 @@
 #include <cstdio>
 #include <memory>
 #include <algorithm>
+#include <unistd.h>
 
 #if defined(__APPLE__)
     //	Include from Frameworks Headers under ProjectBuilder
@@ -96,11 +58,15 @@
 #include <xercesc/util/Platforms/MacOS/MacCarbonFile.hpp>
 #include <xercesc/util/Platforms/MacOS/MacPosixFile.hpp>
 #include <xercesc/util/PanicHandler.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
 
 #if (defined(XML_USE_INMEMORY_MSGLOADER) || defined(XML_USE_INMEM_MESSAGELOADER))
    #include <xercesc/util/MsgLoaders/InMemory/InMemMsgLoader.hpp>
 #endif
-#if (defined(XML_USE_MACOS_UNICODECONVERTER) || defined(XML_USE_NATIVE_TRANSCODER))
+
+#if defined(XML_USE_ICU_TRANSCODER)
+   #include <xercesc/util/Transcoders/ICU/ICUTransService.hpp>
+#elif (defined(XML_USE_MACOS_UNICODECONVERTER) || defined(XML_USE_NATIVE_TRANSCODER))
    #include <xercesc/util/Transcoders/MacOSUnicodeConverter/MacOSUnicodeConverter.hpp>
 #endif
 
@@ -115,6 +81,8 @@
    #include <xercesc/util/NetAccessors/MacOSURLAccessCF/MacOSURLAccessCF.hpp>
 #elif defined(USE_URLACCESS)
    #include <xercesc/util/NetAccessors/MacOSURLAccess/MacOSURLAccess.hpp>
+#elif defined(XML_USE_NETACCESSOR_SOCKET)
+   #include <xercesc/util/NetAccessors/Socket/SocketNetAccessor.hpp>
 #endif
 
 XERCES_CPP_NAMESPACE_BEGIN
@@ -142,6 +110,9 @@ bool	XMLParsePathToFSSpec_Classic(const XMLCh* const pathName, FSSpec& spec, Mem
 //   This flag indicates whether the file system APIs meet our minimum
 //   requirements.
 //
+//  gMacOSXOrBetter
+//   The system version is >= 10.
+//
 // gHasFSSpecAPIs
 //   True if the FSSpecAPIs are available. These are required.
 //
@@ -169,8 +140,12 @@ bool	XMLParsePathToFSSpec_Classic(const XMLCh* const pathName, FSSpec& spec, Mem
 //
 // gUsePosixFiles
 //   True if we're using XMLMacPosixFile rather than XMLMacCarbonFile.
+//
+// gUseGETCWD
+//   True if we can rely on getcwd to get the current directory path.
 //----------------------------------------------------------------------------
 bool gFileSystemCompatible	= false;
+bool gMacOSXOrBetter		= false;
 bool gHasFSSpecAPIs			= false;
 bool gHasFS2TBAPIs			= false;
 bool gHasHFSPlusAPIs		= false;
@@ -178,6 +153,7 @@ bool gHasFSPathAPIs			= false;
 bool gPathAPIsUsePosixPaths	= false;
 bool gHasMPAPIs				= false;
 bool gUsePosixFiles			= false;
+bool gUseGETCWD				= false;
 
 
 // ---------------------------------------------------------------------------
@@ -197,81 +173,88 @@ XMLPlatformUtils::panic(const PanicHandler::PanicReasons reason)
 //  XMLPlatformUtils: File Methods
 // ---------------------------------------------------------------------------
 unsigned int
-XMLPlatformUtils::curFilePos(const FileHandle theFile)
+XMLPlatformUtils::curFilePos(const FileHandle theFile
+                             , MemoryManager* const manager)
 {
 	return reinterpret_cast<XMLMacAbstractFile*>(theFile)->currPos();
 }
 
 void
-XMLPlatformUtils::closeFile(const FileHandle theFile)
+XMLPlatformUtils::closeFile(const FileHandle theFile
+                            , MemoryManager* const manager)
 {
     reinterpret_cast<XMLMacAbstractFile*>(theFile)->close();
+	delete reinterpret_cast<XMLMacAbstractFile*>(theFile);
 }
 
 unsigned int
-XMLPlatformUtils::fileSize(const FileHandle theFile)
+XMLPlatformUtils::fileSize(const FileHandle theFile
+                           , MemoryManager* const manager)
 {
     return reinterpret_cast<XMLMacAbstractFile*>(theFile)->size();
 }
 
 
 FileHandle
-XMLPlatformUtils::openFile(const char* const fileName)
+XMLPlatformUtils::openFile(const char* const fileName
+                           , MemoryManager* const manager)
 {
     // Check to make sure the file system is in a state where we can use it
     if (!gFileSystemCompatible)
-        ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName);
+        ThrowXMLwithMemMgr1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName, manager);
 
-    Janitor<XMLMacAbstractFile> file(XMLMakeMacFile());
+    Janitor<XMLMacAbstractFile> file(XMLMakeMacFile(manager));
     
     return (file->open(fileName, false)) ? file.release() : NULL;
 }
 
 
 FileHandle
-XMLPlatformUtils::openFile(const XMLCh* const fileName)
+XMLPlatformUtils::openFile(const XMLCh* const fileName, MemoryManager* const manager)
 {
     // Check to make sure the file system is in a state where we can use it
     if (!gFileSystemCompatible)
-        ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName);
+        ThrowXMLwithMemMgr1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName, manager);
 
-    Janitor<XMLMacAbstractFile> file(XMLMakeMacFile());
+    Janitor<XMLMacAbstractFile> file(XMLMakeMacFile(manager));
 
     return (file->open(fileName, false)) ? file.release() : NULL;
 }
 
 
 FileHandle
-XMLPlatformUtils::openFileToWrite(const char* const fileName)
+XMLPlatformUtils::openFileToWrite(const char* const fileName
+                                  , MemoryManager* const manager)
 {
     // Check to make sure the file system is in a state where we can use it
     if (!gFileSystemCompatible)
-        ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName);
+        ThrowXMLwithMemMgr1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName, manager);
 
-    Janitor<XMLMacAbstractFile> file(XMLMakeMacFile());
+    Janitor<XMLMacAbstractFile> file(XMLMakeMacFile(manager));
 
     return (file->open(fileName, true)) ? file.release() : NULL;
 }
 
 
 FileHandle
-XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
+XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName
+                                  , MemoryManager* const manager)
 {
     // Check to make sure the file system is in a state where we can use it
     if (!gFileSystemCompatible)
-        ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName);
+        ThrowXMLwithMemMgr1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName, manager);
 
-    Janitor<XMLMacAbstractFile> file(XMLMakeMacFile());
+    Janitor<XMLMacAbstractFile> file(XMLMakeMacFile(manager));
 
     return (file->open(fileName, true)) ? file.release() : NULL;
 }
 
 
 FileHandle
-XMLPlatformUtils::openStdInHandle()
+XMLPlatformUtils::openStdInHandle(MemoryManager* const manager)
 {
     XMLCh stdinStr[] = {chLatin_s, chLatin_t, chLatin_d, chLatin_i, chLatin_n, chNull};
-    ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, stdinStr);
+    ThrowXMLwithMemMgr1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, stdinStr, manager);
     return NULL;
 }
 
@@ -279,7 +262,8 @@ XMLPlatformUtils::openStdInHandle()
 unsigned int
 XMLPlatformUtils::readFileBuffer(   const FileHandle      theFile
                                  ,  const unsigned int    toRead
-                                 ,        XMLByte* const  toFill)
+                                 ,        XMLByte* const  toFill
+                                 ,  MemoryManager* const  manager)
 {
     return reinterpret_cast<XMLMacAbstractFile*>(theFile)->read(toRead, toFill);
 }
@@ -288,14 +272,16 @@ XMLPlatformUtils::readFileBuffer(   const FileHandle      theFile
 void
 XMLPlatformUtils::writeBufferToFile(   const   FileHandle   theFile
                                     ,  const long		    toWrite
-                                    ,  const XMLByte* const toFlush)
+                                    ,  const XMLByte* const toFlush
+                                    ,  MemoryManager* const manager)
 {
     return reinterpret_cast<XMLMacAbstractFile*>(theFile)->write(toWrite, toFlush);
 }
 
 
 void
-XMLPlatformUtils::resetFile(FileHandle theFile)
+XMLPlatformUtils::resetFile(FileHandle theFile
+                            , MemoryManager* const manager)
 {
     reinterpret_cast<XMLMacAbstractFile*>(theFile)->reset();
 }
@@ -332,7 +318,8 @@ XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
 
 
 bool
-XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
+XMLPlatformUtils::isRelative(const XMLCh* const toCheck
+                             , MemoryManager* const manager)
 {
     return (toCheck[0] != L'/');
 }
@@ -342,14 +329,19 @@ XMLCh* XMLPlatformUtils::getCurrentDirectory(MemoryManager* const manager)
 {
 	//	Get a newly allocated path to the current directory
 	FSSpec spec;
+
+	//  Parse to path to determine current directory: this allows the
+	//  path parsing routines to determine best way to find the current
+	//  directory.
+	XMLCh curDirPath[] = { '.', 0 };
 	XMLCh* path =
-		(noErr == FSMakeFSSpec(0, 0, NULL, &spec))
+		(XMLParsePathToFSSpec(curDirPath, spec, manager))
 			? XMLCreateFullPathFromFSSpec(spec, manager)
 			: NULL;
 			
     if (!path)
-		 ThrowXML(XMLPlatformUtilsException,
-	           XMLExcepts::File_CouldNotGetBasePathName);
+		 ThrowXMLwithMemMgr(XMLPlatformUtilsException,
+	           XMLExcepts::File_CouldNotGetBasePathName, manager);
 	           
 	return path;
 }
@@ -400,7 +392,7 @@ XMLPlatformUtils::getCurrentMillis()
 // ---------------------------------------------------------------------------
 
 void*
-XMLPlatformUtils::makeMutex()
+XMLPlatformUtils::makeMutex(MemoryManager*)
 {
 	if (gHasMPAPIs)
 	{
@@ -515,6 +507,11 @@ XMLPlatformUtils::platformInit()
 
 	//	Detect available functions
 	
+	//  Check whether we're on OS X
+	gMacOSXOrBetter			= noErr == Gestalt(gestaltSystemVersion, &value)
+							  && value >= 0x00001000
+							  ;
+	
     //	Look for file system services
     if (noErr == Gestalt(gestaltFSAttr, &value))
     {
@@ -541,9 +538,15 @@ XMLPlatformUtils::platformInit()
 	#if __MSL__ && (__MSL__ < 0x08000 || _MSL_CARBON_FILE_APIS)
 	gUsePosixFiles			= false;
 	#else
-	gUsePosixFiles			= noErr == Gestalt(gestaltSystemVersion, &value)
-							  && value >= 0x00001000
-							  ;
+	gUsePosixFiles			= gMacOSXOrBetter;
+	#endif
+	
+	//  Determine whether to use getcwd or not. We use it only if we're not using MSL,
+	//  and we're on a Mac OS X system.
+	#if __MSL__
+	gUseGETCWD				= false;
+	#else
+	gUseGETCWD				= gMacOSXOrBetter;
 	#endif
 
     //	Look for MP
@@ -579,16 +582,19 @@ XMLPlatformUtils::makeNetAccessor()
     //	XML_USE_NETACCESSOR_URLACCESSCF		-- Use netaccessor based on CFURLAccess (CoreFoundation based)
     //	XML_USE_NETACCESSOR_NATIVE			-- In absence of above selections, chooses URLACCESSCF
     //										   if targetting Carbon, and URLAccess otherwise
+    //	XML_USE_NETACCESSOR_SOCKET			-- Use the sockets based netaccessor
     //
     //	These choices are resolved at the ^^^top^^^ of this file.
 
 #if (defined(USE_URLACCESSCF))
     //	Use the URLAccess code that relies only on CoreFoundation
-	return new MacOSURLAccessCF;
+	return new (fgMemoryManager) MacOSURLAccessCF;
 #elif (defined(USE_URLACCESS))
 	//	Only try to use URLAccess if it's actually available
 	if (URLAccessAvailable())
-		return new MacOSURLAccess;
+		return new (fgMemoryManager) MacOSURLAccess;
+#elif (defined(XML_USE_NETACCESSOR_SOCKET))
+	return new (fgMemoryManager) SocketNetAccessor;
 #endif
 
 	//	No netaccessor available--we can live with it, but you won't
@@ -604,12 +610,25 @@ XMLPlatformUtils::makeNetAccessor()
 XMLMsgLoader*
 XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
 {
+    XMLMsgLoader* retVal;
+    try
+    {
 #if (defined(XML_USE_INMEMORY_MSGLOADER) || defined(XML_USE_INMEM_MESSAGELOADER))
-    return new InMemMsgLoader(msgDomain);
+        retVal = new (fgMemoryManager) InMemMsgLoader(msgDomain);
 #else
-    #error You must provide a message loader
-    return 0;
+        #error You must provide a message loader
+        return 0;
 #endif
+    }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
+    catch(...)
+    {
+         panic(PanicHandler::Panic_CantLoadMsgDomain);
+    }
+    return retVal;
 }
 
 
@@ -622,9 +641,11 @@ XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
 XMLTransService*
 XMLPlatformUtils::makeTransService()
 {
-#if (defined(XML_USE_MACOS_UNICODECONVERTER) || defined(XML_USE_NATIVE_TRANSCODER))
+#if defined (XML_USE_ICU_TRANSCODER)
+	return new (fgMemoryManager) ICUTransService;
+#elif (defined(XML_USE_MACOS_UNICODECONVERTER) || defined(XML_USE_NATIVE_TRANSCODER))
     if (MacOSUnicodeConverter::IsMacOSUnicodeConverterSupported())
-        return new MacOSUnicodeConverter;
+        return new (fgMemoryManager) MacOSUnicodeConverter;
 #else
     #error You must provide a transcoding service implementation
 #endif
@@ -729,14 +750,14 @@ ConvertSlashToColon(char* p, std::size_t charCount)
 //	Factory method to make an appropriate subclass of XMLMacAbstractFile
 //	for our use
 XMLMacAbstractFile*
-XMLMakeMacFile(void)
+XMLMakeMacFile(MemoryManager* manager)
 {
 	XMLMacAbstractFile* result = NULL;
 	
 	if (gUsePosixFiles)
-		result = new XMLMacPosixFile;
+		result = new (manager) XMLMacPosixFile;
 	else
-		result = new XMLMacCarbonFile;
+		result = new (manager) XMLMacCarbonFile;
 		
 	return result;
 }
@@ -794,29 +815,41 @@ XMLParsePathToFSRef_X(const XMLCh* const pathName, FSRef& ref, MemoryManager* co
 	
 	//	If it's a relative path, pre-pend the current directory to the path.
 	//	FSPathMakeRef doesn't deal with relativity on the front of the path
-	if (*p != '/')
+	if (*p != '/' && kMaxMacStaticPathChars > pathLen)
 	{
 		//	Right justify the user path to make room for the pre-pended path
 		std::memmove(p + kMaxMacStaticPathChars - pathLen, p, pathLen);
+		*p = '\0';
 				
 		//	Get the current directory
-        FSSpec spec;
-		if (err == noErr)
-			err = FSMakeFSSpec(0, 0, NULL, &spec);
-        if (err == noErr)
-            err = FSpMakeFSRef(&spec, &ref);
-		
-		//	Get pathname to the current directory
-		if (err == noErr)
-			err = FSRefMakePath(&ref, reinterpret_cast<UInt8*>(p), kMaxMacStaticPathChars - pathLen - 1);	// leave room for one '/'
-		std::size_t prefixLen = std::strlen(p);
-			
-		//	Now munge the two paths back together
-		if (err == noErr)
+		if (gUseGETCWD)
 		{
-			p[prefixLen++] = '/';
-			std::memmove(p + prefixLen, p + kMaxMacStaticPathChars - pathLen, pathLen);
+			//	Get current directory path, leaving room for one '/' after
+			if (err == noErr)
+				getcwd(p, kMaxMacStaticPathChars - pathLen - 1);
 		}
+		else
+		{
+			//	Get current directory path, leaving room for one '/' after
+
+			//	We quiz the carbon file manager for the current directory.
+			//	Note that carbon defaults its concept of the current directory
+			//  to the location of the executable.
+	        FSSpec spec;
+			if (err == noErr)
+				err = FSMakeFSSpec(0, 0, NULL, &spec);
+	        if (err == noErr)
+	            err = FSpMakeFSRef(&spec, &ref);
+			
+			//	Get current directory path, leaving room for one '/' after
+			if (err == noErr)
+				err = FSRefMakePath(&ref, reinterpret_cast<UInt8*>(p), kMaxMacStaticPathChars - pathLen - 1);
+		}
+					
+		//	Now munge the two paths back together
+		std::size_t prefixLen = std::strlen(p);
+		p[prefixLen++] = '/';
+		std::memmove(p + prefixLen, p + kMaxMacStaticPathChars - pathLen, pathLen);
 		
 		//	We now have a path from an absolute starting point
 	}
