@@ -1,68 +1,32 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /**
  * @01A D998714.1 V5R2M0    100301   Swan    :Fix error return flags
  * @02A           V5R2M0    200419   jrhansen : support lowercase function
- * $Id: Iconv400TransService.cpp,v 1.9 2003/05/17 16:32:17 knoaman Exp $
+ * @03A D93234.1  V1R1M0    102804   JRHansen :fix transcode overflow 
+ * @04A D93234.3  V1R1M0    022505   JRHansen :fix transcode error check 
+ * $Id: Iconv400TransService.cpp 568078 2007-08-21 11:43:25Z amassari $
  */
 
 // ---------------------------------------------------------------------------
 //  Includes
 // ---------------------------------------------------------------------------
+#include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/TranscodingException.hpp>
 #include "Iconv400TransService.hpp"
 #include <string.h>
@@ -70,7 +34,6 @@
 #include "iconv_cnv.hpp"
 #include "iconv_util.hpp"
 #include <qusec.h>
-#include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/Janitor.hpp>
 
@@ -85,12 +48,13 @@ XERCES_CPP_NAMESPACE_BEGIN
 //  conversion of all strings. These local helper methods make that easier.
 //
 static UChar* convertToUChar( const   XMLCh* const toConvert
-                            , const unsigned int   srcLen = 0)
+                            , const unsigned int   srcLen = 0
+                            , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager)
 {
     const unsigned int actualLen = srcLen
                                    ? srcLen : XMLString::stringLen(toConvert);
 
-    UChar* tmpBuf = (UChar*) XMLPlatformUtils::fgMemoryManager->allocate
+    UChar* tmpBuf = (UChar*) manager->allocate
     (
         (srcLen + 1) * sizeof(UChar)
     );//new UChar[srcLen + 1];
@@ -420,26 +384,29 @@ Iconv400Transcoder::transcodeFrom(const  XMLByte* const          srcData
         , &err
     );
 
-    if ((err != U_ZERO_ERROR) && (err != U_INDEX_OUTOFBOUNDS_ERROR))
+
+    if ((err != U_ZERO_ERROR) && (err != U_INDEX_OUTOFBOUNDS_ERROR) &&
+	(err != U_BUFFER_OVERFLOW_ERROR))   // @03c
     {
         if (orgTarget != (UChar*)toFill)
             getMemoryManager()->deallocate(orgTarget);//delete [] orgTarget;
 
         if (fFixed)
         {
-            XMLCh tmpBuf[16];
-            XMLString::binToText((unsigned int)(*startTarget), tmpBuf, 16, 16);
-            ThrowXML2
+            XMLCh tmpBuf[17];
+            XMLString::binToText((unsigned int)(*startTarget), tmpBuf, 16, 16, getMemoryManager());
+            ThrowXMLwithMemMgr2
             (
                 TranscodingException
                 , XMLExcepts::Trans_BadSrcCP
                 , tmpBuf
                 , getEncodingName()
+                , getMemoryManager()
             );
         }
          else
         {
-            ThrowXML(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
+            ThrowXMLwithMemMgr(TranscodingException, XMLExcepts::Trans_BadSrcSeq, getMemoryManager());
         }
     }
 
@@ -456,7 +423,7 @@ Iconv400Transcoder::transcodeFrom(const  XMLByte* const          srcData
     //
     if (fFixed)
     {
-        const unsigned char fillSize = (unsigned char)ucnv_getMaxCharSize(fConverter);;
+        const unsigned char fillSize = (unsigned char)ucnv_getMaxCharSize(fConverter);
         memset(charSizes, fillSize, maxChars);
     }
      else
@@ -530,10 +497,10 @@ Iconv400Transcoder::transcodeTo( const   XMLCh* const    srcData
     }
      else
     {
-        tmpBufPtr = convertToUChar(srcData, srcCount);
+        tmpBufPtr = convertToUChar(srcData, srcCount, getMemoryManager());
         srcPtr = tmpBufPtr;
     }
-    ArrayJanitor<UChar> janTmpBuf(tmpBufPtr, XMLPlatformUtils::fgMemoryManager);
+    ArrayJanitor<UChar> janTmpBuf(tmpBufPtr, getMemoryManager());
 
     //
     //  Set the appropriate callback so that it will either fail or use
@@ -564,14 +531,15 @@ Iconv400Transcoder::transcodeTo( const   XMLCh* const    srcData
 
     if (err)   /*@01A*/
     {
-        XMLCh tmpBuf[16];
-        XMLString::binToText((unsigned int)*startSrc, tmpBuf, 16, 16);
-        ThrowXML2
+        XMLCh tmpBuf[17];
+        XMLString::binToText((unsigned int)*startSrc, tmpBuf, 16, 16, getMemoryManager());
+        ThrowXMLwithMemMgr2
         (
             TranscodingException
             , XMLExcepts::Trans_Unrepresentable
             , tmpBuf
             , getEncodingName()
+            , getMemoryManager()
         );
     }
 
@@ -652,7 +620,8 @@ Iconv400LCPTranscoder::~Iconv400LCPTranscoder()
 // ---------------------------------------------------------------------------
 //  Iconv400LCPTranscoder: Constructors and Destructor
 // ---------------------------------------------------------------------------
-unsigned int Iconv400LCPTranscoder::calcRequiredSize(const XMLCh* const srcText)
+unsigned int Iconv400LCPTranscoder::calcRequiredSize(const XMLCh* const srcText
+                                                     , MemoryManager* const manager)
 {
     if (!srcText)
         return 0;
@@ -679,7 +648,8 @@ unsigned int Iconv400LCPTranscoder::calcRequiredSize(const XMLCh* const srcText)
     return (unsigned int)targetCap;
 }
 
-unsigned int Iconv400LCPTranscoder::calcRequiredSize(const char* const srcText)
+unsigned int Iconv400LCPTranscoder::calcRequiredSize(const char* const srcText
+                                                     , MemoryManager* const manager)
 {
     if (!srcText)
         return 0;
@@ -891,10 +861,11 @@ XMLCh* Iconv400LCPTranscoder::transcode(const char* const toTranscode)
             , &err
         );
 
-        if (err != U_BUFFER_OVERFLOW_ERROR)
+        if (err == U_BUFFER_OVERFLOW_ERROR)  // @04c
 	{
 
         err = U_ZERO_ERROR;
+	delete [] retVal;  // @04a
         retVal = new XMLCh[targetCap];
         ucnv_toUChars
         (
@@ -963,10 +934,11 @@ XMLCh* Iconv400LCPTranscoder::transcode(const char* const toTranscode,
             , &err
         );
 
-        if (err != U_BUFFER_OVERFLOW_ERROR)
+        if (err == U_BUFFER_OVERFLOW_ERROR)  // @04c
 	{
 
         err = U_ZERO_ERROR;
+        manager->deallocate(retVal);  //@04a
         retVal = (XMLCh*) manager->allocate(targetCap * sizeof(XMLCh));//new XMLCh[targetCap];
         ucnv_toUChars
         (
@@ -993,7 +965,8 @@ XMLCh* Iconv400LCPTranscoder::transcode(const char* const toTranscode,
 
 bool Iconv400LCPTranscoder::transcode(const  char* const     toTranscode
                                 ,       XMLCh* const    toFill
-                                , const unsigned int    maxChars)
+                                , const unsigned int    maxChars
+                                , MemoryManager* const  manager)
 {
     // Check for a couple of psycho corner cases
     if (!toTranscode || !maxChars)
@@ -1033,7 +1006,8 @@ bool Iconv400LCPTranscoder::transcode(const  char* const     toTranscode
 
 bool Iconv400LCPTranscoder::transcode(   const   XMLCh* const    toTranscode
                                     ,       char* const     toFill
-                                    , const unsigned int    maxChars)
+                                    , const unsigned int    maxChars
+                                    , MemoryManager* const  manager)
 {
     // Watch for a few psycho corner cases
     if (!toTranscode || !maxChars)
