@@ -1,61 +1,22 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Id: XMLException.cpp,v 1.6 2003/05/15 19:07:46 knoaman Exp $
+ * $Id: XMLException.cpp 568078 2007-08-21 11:43:25Z amassari $
  */
 
 
@@ -69,33 +30,17 @@
 #include <xercesc/util/XMLRegisterCleanup.hpp>
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
+#include <xercesc/util/XMLInitializer.hpp>
 
 
 XERCES_CPP_NAMESPACE_BEGIN
 
-// ---------------------------------------------------------------------------
-//  Local data
-//
-//  gDefErrMsg
-//      For now, if we cannot load an exception message, we load up the
-//      default.
-// ---------------------------------------------------------------------------
-static const XMLCh  gDefErrMsg[] =
-{
-        chLatin_C, chLatin_o, chLatin_u, chLatin_l, chLatin_d
-    ,   chSpace, chLatin_n, chLatin_o, chLatin_t, chSpace
-    ,   chLatin_l, chLatin_o, chLatin_a, chLatin_d, chSpace
-    ,   chLatin_m, chLatin_e, chLatin_s, chLatin_s, chLatin_a
-    ,   chLatin_g, chLatin_e, chNull
-};
-
-
 // ---------------------------------------------------------------------------
 //  Local, static data
 // ---------------------------------------------------------------------------
 static XMLMsgLoader* sMsgLoader = 0;
 static XMLRegisterCleanup msgLoaderCleanup;
-
+static bool sScannerMutexRegistered = false;
 static XMLMutex* sMsgMutex = 0;
 static XMLRegisterCleanup msgMutexCleanup;
 
@@ -109,21 +54,16 @@ static XMLRegisterCleanup msgMutexCleanup;
 //
 static XMLMutex& gMsgMutex()
 {
-
-    if (!sMsgMutex)
+    if (!sScannerMutexRegistered)
     {
-        XMLMutex* tmpMutex = new XMLMutex;
-        if (XMLPlatformUtils::compareAndSwap((void**)&sMsgMutex, tmpMutex, 0))
-        {
-            // Some other thread beat us to it, so let's clean up ours.
-            delete tmpMutex;
-        }
-        else
-        {
-            // This is the real mutex.  Register it for cleanup at Termination.
-			msgMutexCleanup.registerCleanup(XMLException::reinitMsgMutex);
-        }
+        XMLMutexLock lockInit(XMLPlatformUtils::fgAtomicMutex);
 
+        if (!sScannerMutexRegistered)
+        {
+            sMsgMutex = new XMLMutex(XMLPlatformUtils::fgMemoryManager);
+            msgMutexCleanup.registerCleanup(XMLException::reinitMsgMutex);
+            sScannerMutexRegistered = true;
+        }
     }
 
     return *sMsgMutex;
@@ -135,34 +75,44 @@ static XMLMutex& gMsgMutex()
 //
 static XMLMsgLoader& gGetMsgLoader()
 {
-
-    // Lock the message loader mutex and load the text
-	XMLMutexLock lockInit(&gMsgMutex());
-
-    // Fault it in on first request
     if (!sMsgLoader)
     {
-        sMsgLoader = XMLPlatformUtils::loadMsgSet(XMLUni::fgExceptDomain);
-        if (!sMsgLoader)
-            XMLPlatformUtils::panic(PanicHandler::Panic_CantLoadMsgDomain);
+        // Lock the message loader mutex and load the text
+	    XMLMutexLock lockInit(&gMsgMutex());
 
-        //
-        // Register this XMLMsgLoader for cleanup at Termination.
-        //
-        msgLoaderCleanup.registerCleanup(XMLException::reinitMsgLoader);
+        // Fault it in on first request
+        if (!sMsgLoader)
+        {
+            sMsgLoader = XMLPlatformUtils::loadMsgSet(XMLUni::fgExceptDomain);
+            if (!sMsgLoader)
+                XMLPlatformUtils::panic(PanicHandler::Panic_CantLoadMsgDomain);
+
+            //
+            // Register this XMLMsgLoader for cleanup at Termination.
+            //
+            msgLoaderCleanup.registerCleanup(XMLException::reinitMsgLoader);
+        }
     }
 
     // We got it, so return it
     return *sMsgLoader;
 }
 
+void XMLInitializer::initializeExceptionMsgLoader()
+{
+    sMsgLoader = XMLPlatformUtils::loadMsgSet(XMLUni::fgExceptDomain);
+    if (sMsgLoader) {
+        msgLoaderCleanup.registerCleanup(XMLException::reinitMsgLoader);
+    }
+}
+
 // ---------------------------------------------------------------------------
 //  XMLException: Virtual destructor
 // ---------------------------------------------------------------------------
 XMLException::~XMLException()
 {
-    XMLPlatformUtils::fgMemoryManager->deallocate(fMsg);
-    XMLPlatformUtils::fgMemoryManager->deallocate(fSrcFile);
+    fMemoryManager->deallocate(fMsg);
+    fMemoryManager->deallocate(fSrcFile);
 }
 
 
@@ -172,8 +122,8 @@ XMLException::~XMLException()
 void XMLException::setPosition(const char* const file, const unsigned int line)
 {
     fSrcLine = line;
-	XMLPlatformUtils::fgMemoryManager->deallocate(fSrcFile);
-    fSrcFile = XMLString::replicate(file, XMLPlatformUtils::fgMemoryManager);
+	fMemoryManager->deallocate(fSrcFile);
+    fSrcFile = XMLString::replicate(file, fMemoryManager);
 }
 
 
@@ -186,34 +136,40 @@ XMLException::XMLException() :
     , fSrcFile(0)
     , fSrcLine(0)
     , fMsg(0)
+    , fMemoryManager(XMLPlatformUtils::fgMemoryManager)
 {
 }
 
 
 XMLException::XMLException( const   char* const     srcFile
-                            , const unsigned int    srcLine) :
+                            , const unsigned int    srcLine
+                            , MemoryManager* const  memoryManager) :
 
     fCode(XMLExcepts::NoError)
     , fSrcFile(0)
     , fSrcLine(srcLine)
     , fMsg(0)
+    , fMemoryManager(memoryManager)
 {
-    fSrcFile = XMLString::replicate(srcFile, XMLPlatformUtils::fgMemoryManager);
+    if (!memoryManager)
+        fMemoryManager = XMLPlatformUtils::fgMemoryManager;
+    fSrcFile = XMLString::replicate(srcFile, fMemoryManager);
 }
 
 
 XMLException::XMLException(const XMLException& toCopy) :
-
-    fCode(toCopy.fCode)
+    XMemory(toCopy)
+    , fCode(toCopy.fCode)
     , fSrcFile(0)
     , fSrcLine(toCopy.fSrcLine)
-    , fMsg(XMLString::replicate(toCopy.fMsg, XMLPlatformUtils::fgMemoryManager))
+    , fMsg(XMLString::replicate(toCopy.fMsg, toCopy.fMemoryManager))
+    , fMemoryManager(toCopy.fMemoryManager)        
 {
     if (toCopy.fSrcFile) {
         fSrcFile = XMLString::replicate
         (
             toCopy.fSrcFile
-            , XMLPlatformUtils::fgMemoryManager
+            , fMemoryManager
         );
     }
 }
@@ -223,11 +179,13 @@ XMLException& XMLException::operator=(const XMLException& toAssign)
 {
     if (this != &toAssign)
     {
-        XMLPlatformUtils::fgMemoryManager->deallocate(fSrcFile);
+        //use the original memory manager to deallocate
+        fMemoryManager->deallocate(fSrcFile);
         fSrcFile = 0;
-        XMLPlatformUtils::fgMemoryManager->deallocate(fMsg);
+        fMemoryManager->deallocate(fMsg);
         fMsg = 0;
 
+        fMemoryManager = toAssign.fMemoryManager;
         fSrcLine = toAssign.fSrcLine;
         fCode = toAssign.fCode;
 
@@ -235,7 +193,7 @@ XMLException& XMLException::operator=(const XMLException& toAssign)
             fMsg = XMLString::replicate
             (
                 toAssign.fMsg
-                , XMLPlatformUtils::fgMemoryManager
+                , fMemoryManager
             );
         }
 
@@ -243,7 +201,7 @@ XMLException& XMLException::operator=(const XMLException& toAssign)
             fSrcFile = XMLString::replicate
             (
                 toAssign.fSrcFile
-                , XMLPlatformUtils::fgMemoryManager
+                , fMemoryManager
             );
         }
     }
@@ -269,14 +227,14 @@ void XMLException::loadExceptText(const XMLExcepts::Codes toLoad)
 	{
 		fMsg = XMLString::replicate
         (
-            gDefErrMsg
-            , XMLPlatformUtils::fgMemoryManager
+        XMLUni::fgDefErrMsg
+            , fMemoryManager
         );
 		return;
 	}
 
     // We got the text so replicate it into the message member
-    fMsg = XMLString::replicate(errText, XMLPlatformUtils::fgMemoryManager);
+    fMsg = XMLString::replicate(errText, fMemoryManager);
 }
 
 
@@ -295,18 +253,18 @@ XMLException::loadExceptText(const  XMLExcepts::Codes toLoad
     XMLCh errText[msgSize + 1];
 
     // load the text
-	if (!gGetMsgLoader().loadMsg(toLoad, errText, msgSize, text1, text2, text3, text4))
+	if (!gGetMsgLoader().loadMsg(toLoad, errText, msgSize, text1, text2, text3, text4, fMemoryManager))
 	{
 		fMsg = XMLString::replicate
         (
-            gDefErrMsg
-            , XMLPlatformUtils::fgMemoryManager
+        XMLUni::fgDefErrMsg
+            , fMemoryManager
         );
 		return;
 	}
 
     // We got the text so replicate it into the message member
-    fMsg = XMLString::replicate(errText, XMLPlatformUtils::fgMemoryManager);
+    fMsg = XMLString::replicate(errText, fMemoryManager);
 }
 
 
@@ -325,18 +283,18 @@ XMLException::loadExceptText(const  XMLExcepts::Codes toLoad
     XMLCh errText[msgSize + 1];
 
     // load the text
-	if (!gGetMsgLoader().loadMsg(toLoad, errText, msgSize, text1, text2, text3, text4))
+	if (!gGetMsgLoader().loadMsg(toLoad, errText, msgSize, text1, text2, text3, text4, fMemoryManager))
 	{
 		fMsg = XMLString::replicate
         (
-            gDefErrMsg
-            , XMLPlatformUtils::fgMemoryManager
+        XMLUni::fgDefErrMsg
+            , fMemoryManager
         );
 		return;
 	}
 
     // We got the text so replicate it into the message member
-    fMsg = XMLString::replicate(errText, XMLPlatformUtils::fgMemoryManager);
+    fMsg = XMLString::replicate(errText, fMemoryManager);
 }
 
 // -----------------------------------------------------------------------
@@ -344,8 +302,9 @@ XMLException::loadExceptText(const  XMLExcepts::Codes toLoad
 // -----------------------------------------------------------------------
 void XMLException::reinitMsgMutex()
 {
-	delete sMsgMutex;
-	sMsgMutex = 0;
+    delete sMsgMutex;
+    sMsgMutex = 0;
+    sScannerMutexRegistered = false;
 }
 
 // -----------------------------------------------------------------------
@@ -353,8 +312,8 @@ void XMLException::reinitMsgMutex()
 // -----------------------------------------------------------------------
 void XMLException::reinitMsgLoader()
 {
-	delete sMsgLoader;
-	sMsgLoader = 0;
+    delete sMsgLoader;
+    sMsgLoader = 0;
 }
 
 XERCES_CPP_NAMESPACE_END
