@@ -1,103 +1,22 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2001-2002 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 2001, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Log: BlockRangeFactory.cpp,v $
- * Revision 1.4  2002/11/04 15:17:00  tng
- * C++ Namespace Support.
- *
- * Revision 1.3  2002/09/24 19:59:36  tng
- * Performance: use XMLString::equals instead of XMLString::compareString
- *
- * Revision 1.2  2002/02/05 13:20:06  tng
- * [Bug 5716] Can't parse with Validation more than one file.
- *
- * Revision 1.1.1.1  2002/02/01 22:22:28  peiyongz
- * sane_include
- *
- * Revision 1.6  2001/11/09 21:27:21  tng
- * Schema Regx Fix: manually add the extra specials and private use block range.
- *
- * Revision 1.5  2001/11/09 18:10:09  tng
- * Schema Regx: Block name 'X' should begin with 'IsX'.  And add missing block names and range.
- *
- * Revision 1.4  2001/06/22 16:26:27  peiyongz
- * fix: redefinition of SIZE
- *
- * Revision 1.3  2001/05/11 13:26:40  tng
- * Copyright update.
- *
- * Revision 1.2  2001/05/03 18:17:21  knoaman
- * Some design changes:
- * o Changed the TokenFactory from a single static instance, to a
- *    normal class. Each RegularExpression object will have its own
- *    instance of TokenFactory, and that instance will be passed to
- *    other classes that need to use a TokenFactory to create Token
- *    objects (with the exception of RangeTokenMap).
- * o Added a new class RangeTokenMap to map a the different ranges
- *    in a given category to a specific RangeFactory object. In the old
- *    design RangeFactory had dual functionality (act as a Map, and as
- *    a factory for creating RangeToken(s)). The RangeTokenMap will
- *    have its own copy of the TokenFactory. There will be only one
- *    instance of the RangeTokenMap class, and that instance will be
- *    lazily deleted when XPlatformUtils::Terminate is called.
- *
- * Revision 1.1  2001/03/02 19:26:41  knoaman
- * Schema: Regular expression handling part II
- *
+ * $Id: BlockRangeFactory.cpp 568078 2007-08-21 11:43:25Z amassari $
  */
 
 // ---------------------------------------------------------------------------
@@ -115,13 +34,13 @@ XERCES_CPP_NAMESPACE_BEGIN
 // ---------------------------------------------------------------------------
 //  Local static data
 // ---------------------------------------------------------------------------
-const int   BLOCKNAMESIZE = 96;
+const int   BLOCKNAMESIZE = 93;
 
 // Block Names IsX
 // only define Specials as FEFF..FEFF, missing Specials as FFF0..FFFD, add manually
 // only define private use as E000..F8FF,
 //    missing 2 private use (F0000..FFFFD and 100000..10FFFD), add manually
-const XMLCh fgBlockNames[][BLOCKNAMESIZE] =
+const XMLCh fgBlockNames[][50] =
 {
     { chLatin_I, chLatin_s, chLatin_B, chLatin_a, chLatin_s, chLatin_i, chLatin_c, chLatin_L, chLatin_a,
       chLatin_t, chLatin_i, chLatin_n,  chNull },
@@ -292,14 +211,6 @@ const XMLCh fgBlockNames[][BLOCKNAMESIZE] =
     { chLatin_I, chLatin_s, chLatin_H, chLatin_a, chLatin_n, chLatin_g, chLatin_u, chLatin_l, chLatin_S,
       chLatin_y, chLatin_l, chLatin_l, chLatin_a, chLatin_b, chLatin_l, chLatin_e, chLatin_s,
        chNull },
-    { chLatin_I, chLatin_s, chLatin_H, chLatin_i, chLatin_g, chLatin_h, chLatin_S, chLatin_u, chLatin_r,
-      chLatin_r, chLatin_o, chLatin_g, chLatin_a, chLatin_t, chLatin_e, chLatin_s,  chNull },
-    { chLatin_I, chLatin_s, chLatin_H, chLatin_i, chLatin_g, chLatin_h, chLatin_P, chLatin_r, chLatin_i,
-      chLatin_v, chLatin_a, chLatin_t, chLatin_e, chLatin_U, chLatin_s, chLatin_e,
-      chLatin_S, chLatin_u, chLatin_r, chLatin_r, chLatin_o, chLatin_g, chLatin_a,
-      chLatin_t, chLatin_e, chLatin_s,  chNull },
-    { chLatin_I, chLatin_s, chLatin_L, chLatin_o, chLatin_w, chLatin_S, chLatin_u, chLatin_r, chLatin_r,
-      chLatin_o, chLatin_g, chLatin_a, chLatin_t, chLatin_e, chLatin_s,  chNull },
     { chLatin_I, chLatin_s, chLatin_P, chLatin_r, chLatin_i, chLatin_v, chLatin_a, chLatin_t, chLatin_e,
       chLatin_U, chLatin_s, chLatin_e,  chNull },
     { chLatin_I, chLatin_s, chLatin_C, chLatin_J, chLatin_K, chLatin_C, chLatin_o, chLatin_m, chLatin_p,
@@ -373,19 +284,16 @@ const XMLInt32 blockRanges[] =
     0x2600,0x26FF,0x2700,0x27BF,0x2800,0x28FF,0x2E80,0x2EFF,0x2F00,0x2FDF,0x2FF0,0x2FFF,
     0x3000,0x303F,0x3040,0x309F,0x30A0,0x30FF,0x3100,0x312F,0x3130,0x318F,0x3190,0x319F,
     0x31A0,0x31BF,0x3200,0x32FF,0x3300,0x33FF,0x3400,0x4DB5,0x4E00,0x9FFF,0xA000,0xA48F,
-    0xA490,0xA4CF,0xAC00,0xD7A3,0xD800,0xDB7F,0xDB80,0xDBFF,0xDC00,0xDFFF,0xE000,0xF8FF,
-    0xF900,0xFAFF,0xFB00,0xFB4F,0xFB50,0xFDFF,0xFE20,0xFE2F,0xFE30,0xFE4F,0xFE50,0xFE6F,
-    0xFE70,0xFEFE,0xFEFF,0xFEFF,0xFF00,0xFFEF,0x10300,0x1032F,0x10330,0x1034F,
-    0x10400,0x1044F,0x1D000,0x1D0FF,0x1D100,0x1D1FF,0x1D400,0x1D7FF,0x20000,0x2A6D6,0x2F800,0x2FA1F,
-    0xE0000,0xE007F, chNull
+    0xA490,0xA4CF,0xAC00,0xD7A3,0xE000,0xF8FF,0xF900,0xFAFF,0xFB00,0xFB4F,0xFB50,0xFDFF,
+    0xFE20,0xFE2F,0xFE30,0xFE4F,0xFE50,0xFE6F,0xFE70,0xFEFE,0xFEFF,0xFEFF,0xFF00,0xFFEF,
+    0x10300,0x1032F,0x10330,0x1034F,0x10400,0x1044F,0x1D000,0x1D0FF,0x1D100,0x1D1FF,
+    0x1D400,0x1D7FF,0x20000,0x2A6D6,0x2F800,0x2FA1F,0xE0000,0xE007F, chNull
 };
 
 // ---------------------------------------------------------------------------
 //  BlockRangeFactory: Constructors and Destructor
 // ---------------------------------------------------------------------------
-BlockRangeFactory::BlockRangeFactory() :
-   fRangesCreated(false)
- , fKeywordsInitialized(false)
+BlockRangeFactory::BlockRangeFactory()
 {
 
 }
@@ -397,16 +305,15 @@ BlockRangeFactory::~BlockRangeFactory() {
 // ---------------------------------------------------------------------------
 //  BlockRangeFactory: Range creation methods
 // ---------------------------------------------------------------------------
-void BlockRangeFactory::buildRanges() {
+void BlockRangeFactory::buildRanges(RangeTokenMap *rangeTokMap) {
 
     if (fRangesCreated)
         return;
 
     if (!fKeywordsInitialized) {
-        initializeKeywordMap();
+        initializeKeywordMap(rangeTokMap);
     }
 
-    RangeTokenMap* rangeTokMap = RangeTokenMap::instance();
     TokenFactory* tokFactory = rangeTokMap->getTokenFactory();
 
     //for performance, once the desired specials and private use are found
@@ -427,7 +334,14 @@ void BlockRangeFactory::buildRanges() {
             tok->addRange(0x100000, 0x10FFFD);
             foundPrivate = true;
         }
+
+        // Build the internal map.
+        tok->createMap();
         rangeTokMap->setRangeToken(fgBlockNames[i], tok);
+        tok = (RangeToken*) RangeToken::complementRanges(tok, tokFactory);
+        // Build the internal map.
+        tok->createMap();
+        rangeTokMap->setRangeToken(fgBlockNames[i], tok , true);
     }
 
     fRangesCreated = true;
@@ -436,13 +350,11 @@ void BlockRangeFactory::buildRanges() {
 // ---------------------------------------------------------------------------
 //  BlockRangeFactory: Range creation methods
 // ---------------------------------------------------------------------------
-void BlockRangeFactory::initializeKeywordMap() {
+void BlockRangeFactory::initializeKeywordMap(RangeTokenMap *rangeTokMap) {
 
     if (fKeywordsInitialized)
         return;
 
-	RangeTokenMap* rangeTokMap = RangeTokenMap::instance();
-
 	for (int i=0; i< BLOCKNAMESIZE; i++) {
         rangeTokMap->addKeywordMap(fgBlockNames[i], fgBlockCategory);
     }
