@@ -1,9 +1,10 @@
 /*
- * Copyright 2001,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -15,67 +16,7 @@
  */
 
 /*
- * $Log: RangeTokenMap.cpp,v $
- * Revision 1.11  2004/09/08 13:56:47  peiyongz
- * Apache License Version 2.0
- *
- * Revision 1.10  2004/07/22 15:37:18  knoaman
- * Use file static instance instead of local static instance
- *
- * Revision 1.9  2004/01/13 16:17:10  knoaman
- * Fo sanity, use class name to qualify method
- *
- * Revision 1.8  2004/01/09 22:41:58  knoaman
- * Use a global static mutex for locking when creating local static mutexes instead of compareAndSwap
- *
- * Revision 1.7  2003/12/17 00:18:37  cargilld
- * Update to memory management so that the static memory manager (one used to call Initialize) is only for static data.
- *
- * Revision 1.6  2003/10/17 16:44:34  knoaman
- * Fix multithreading problem.
- *
- * Revision 1.5  2003/05/18 14:02:06  knoaman
- * Memory manager implementation: pass per instance manager.
- *
- * Revision 1.4  2003/05/15 19:10:23  knoaman
- * Add missing include.
- *
- * Revision 1.3  2003/03/04 21:11:12  knoaman
- * [Bug 17516] Thread safety problems in ../util/ and ../util/regx.
- *
- * Revision 1.2  2002/11/04 15:17:00  tng
- * C++ Namespace Support.
- *
- * Revision 1.1.1.1  2002/02/01 22:22:30  peiyongz
- * sane_include
- *
- * Revision 1.5  2001/10/25 15:06:26  tng
- * Thread safe the static instance.
- *
- * Revision 1.4  2001/10/23 23:13:41  peiyongz
- * [Bug#880] patch to PlatformUtils:init()/term() and related. from Mark Weaver
- *
- * Revision 1.3  2001/07/16 21:28:25  knoaman
- * fix bug - no delete for the static instance in destructor.
- *
- * Revision 1.2  2001/05/11 13:26:45  tng
- * Copyright update.
- *
- * Revision 1.1  2001/05/03 18:17:40  knoaman
- * Some design changes:
- * o Changed the TokenFactory from a single static instance, to a
- *    normal class. Each RegularExpression object will have its own
- *    instance of TokenFactory, and that instance will be passed to
- *    other classes that need to use a TokenFactory to create Token
- *    objects (with the exception of RangeTokenMap).
- * o Added a new class RangeTokenMap to map a the different ranges
- *    in a given category to a specific RangeFactory object. In the old
- *    design RangeFactory had dual functionality (act as a Map, and as
- *    a factory for creating RangeToken(s)). The RangeTokenMap will
- *    have its own copy of the TokenFactory. There will be only one
- *    instance of the RangeTokenMap class, and that instance will be
- *    lazily deleted when XPlatformUtils::Terminate is called.
- *
+ * $Id: RangeTokenMap.cpp 568078 2007-08-21 11:43:25Z amassari $
  */
 
 // ---------------------------------------------------------------------------
@@ -85,11 +26,16 @@
 #include <xercesc/util/regx/RangeToken.hpp>
 #include <xercesc/util/regx/RegxDefs.hpp>
 #include <xercesc/util/regx/TokenFactory.hpp>
-#include <xercesc/util/regx/RangeFactory.hpp>
+#include <xercesc/util/regx/XMLRangeFactory.hpp>
+#include <xercesc/util/regx/ASCIIRangeFactory.hpp>
+#include <xercesc/util/regx/UnicodeRangeFactory.hpp>
+#include <xercesc/util/regx/BlockRangeFactory.hpp>
 #include <xercesc/util/PlatformUtils.hpp>
 #include <xercesc/util/XMLExceptMsgs.hpp>
 #include <xercesc/util/XMLRegisterCleanup.hpp>
 #include <xercesc/util/StringPool.hpp>
+#include <xercesc/util/XMLInitializer.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -118,7 +64,7 @@ static XMLMutex& getRangeTokMapMutex()
         // If we got here first, then register it and set the registered flag
         if (!sRangeTokMapMutex)
         {
-            sRangeTokMapMutex = new XMLMutex;
+            sRangeTokMapMutex = new XMLMutex(XMLPlatformUtils::fgMemoryManager);
             rangeTokMapRegistryCleanup.registerCleanup(reinitRangeTokMapMutex);
         }
     }
@@ -130,6 +76,16 @@ static XMLMutex& getRangeTokMapMutex()
 // ---------------------------------------------------------------------------
 RangeTokenMap* RangeTokenMap::fInstance = 0;
 
+void XMLInitializer::initializeRangeTokenMap()
+{
+    RangeTokenMap::fInstance = new RangeTokenMap(XMLPlatformUtils::fgMemoryManager);
+    if (RangeTokenMap::fInstance)
+    {
+        rangeTokMapInstanceCleanup.registerCleanup(RangeTokenMap::reinitInstance);
+        RangeTokenMap::fInstance->buildTokenRanges();
+    }
+}
+
 
 // ---------------------------------------------------------------------------
 //  RangeTokenElemMap: Constructors and Destructor
@@ -150,27 +106,38 @@ RangeTokenElemMap::~RangeTokenElemMap()
 // ---------------------------------------------------------------------------
 //  RangeTokenMap: Constructors and Destructor
 // ---------------------------------------------------------------------------
-RangeTokenMap::RangeTokenMap() :
-    fRegistryInitialized(false)
-    , fTokenRegistry(0)
+
+typedef JanitorMemFunCall<RangeTokenMap>    CleanupType;
+
+RangeTokenMap::RangeTokenMap(MemoryManager* manager) :
+    fTokenRegistry(0)
     , fRangeMap(0)
     , fCategories(0)
-    , fTokenFactory(0) {
+    , fTokenFactory(0)
+    , fMutex(manager)
+{
+    CleanupType cleanup(this, &RangeTokenMap::cleanUp);
 
+    try {
+        fTokenRegistry = new (manager) RefHashTableOf<RangeTokenElemMap>(109, manager);
+        fRangeMap = new (manager) RefHashTableOf<RangeFactory>(29, manager);
+        fCategories = new (manager) XMLStringPool(109, manager);
+        fTokenFactory = new (manager) TokenFactory(manager);
+        initializeRegistry();
+    }
+    catch(const OutOfMemoryException&)
+    {
+        cleanup.release();
+
+        throw;
+    }
+
+    cleanup.release();
 }
 
 RangeTokenMap::~RangeTokenMap() {
 
-    delete fTokenRegistry;
-    fTokenRegistry = 0;
-
-    delete fRangeMap;
-    fRangeMap = 0;
-
-    delete fCategories;
-    fCategories = 0;
-    delete fTokenFactory;
-    fTokenFactory = 0;
+    cleanUp();
 }
 
 // ---------------------------------------------------------------------------
@@ -179,9 +146,6 @@ RangeTokenMap::~RangeTokenMap() {
 RangeToken* RangeTokenMap::getRange(const XMLCh* const keyword,
 								    const bool complement) {
 
-    if (fTokenRegistry == 0 || fRangeMap == 0 || fCategories == 0)
-        return 0;
-
     if (!fTokenRegistry->containsKey(keyword))
         return 0;
 
@@ -197,24 +161,25 @@ RangeToken* RangeTokenMap::getRange(const XMLCh* const keyword,
 
         if (!rangeTok)
         {
-            rangeTok = elemMap->getRangeToken();
-            if (!rangeTok)
+            unsigned int categId = elemMap->getCategoryId();
+            const XMLCh* categName = fCategories->getValueForId(categId);
+            RangeFactory* rangeFactory = fRangeMap->get(categName);
+
+            if (rangeFactory)
             {
-                unsigned int categId = elemMap->getCategoryId();
-                const XMLCh* categName = fCategories->getValueForId(categId);
-                RangeFactory* rangeFactory = fRangeMap->get(categName);
+                rangeFactory->buildRanges(this);
+                rangeTok = elemMap->getRangeToken(complement);
 
-                if (rangeFactory == 0)
-                    return 0;
-
-                rangeFactory->buildRanges();
-                rangeTok = elemMap->getRangeToken();
-            }
-
-            if (complement)
-            {
-                rangeTok = (RangeToken*) RangeToken::complementRanges(rangeTok, fTokenFactory, fTokenRegistry->getMemoryManager());
-                elemMap->setRangeToken(rangeTok , complement);
+                // see if we are complementing an existing range
+                if (!rangeTok && complement)
+                {
+                    rangeTok = elemMap->getRangeToken();
+                    if (rangeTok)
+                    {
+                        rangeTok = (RangeToken*) RangeToken::complementRanges(rangeTok, fTokenFactory, fTokenRegistry->getMemoryManager());
+                        elemMap->setRangeToken(rangeTok , complement);
+                    }
+                }
             }
         }
     }
@@ -228,23 +193,18 @@ RangeToken* RangeTokenMap::getRange(const XMLCh* const keyword,
 // ---------------------------------------------------------------------------
 void RangeTokenMap::addCategory(const XMLCh* const categoryName) {
 
-    if (fCategories)
-	    fCategories->addOrFind(categoryName);
+    fCategories->addOrFind(categoryName);
 }
 
 void RangeTokenMap::addRangeMap(const XMLCh* const categoryName,
                                 RangeFactory* const rangeFactory) {
 
-    if (fRangeMap)
-	    fRangeMap->put((void*)categoryName, rangeFactory);
+    fRangeMap->put((void*)categoryName, rangeFactory);
 }
 
 void RangeTokenMap::addKeywordMap(const XMLCh* const keyword,
                                  const XMLCh* const categoryName) {
 
-    if (fCategories == 0 || fTokenRegistry == 0)
-        return;
-
 	unsigned int categId = fCategories->getId(categoryName);
 
 	if (categId == 0) {
@@ -270,9 +230,6 @@ void RangeTokenMap::addKeywordMap(const XMLCh* const keyword,
 void RangeTokenMap::setRangeToken(const XMLCh* const keyword,
                                   RangeToken* const tok,const bool complement) {
 
-    if (fTokenRegistry == 0)
-		return;
-
 	if (fTokenRegistry->containsKey(keyword)) {
         fTokenRegistry->get(keyword)->setRangeToken(tok, complement);
     }
@@ -287,21 +244,48 @@ void RangeTokenMap::setRangeToken(const XMLCh* const keyword,
 // ---------------------------------------------------------------------------
 void RangeTokenMap::initializeRegistry() {
 
-    if (!fRegistryInitialized)
-    {
-        XMLMutexLock lockInit(&fMutex);
+    // Add categories
+    fCategories->addOrFind(fgXMLCategory);
+    fCategories->addOrFind(fgASCIICategory);
+    fCategories->addOrFind(fgUnicodeCategory);
+    fCategories->addOrFind(fgBlockCategory);
 
-        if (!fRegistryInitialized)
-        {
-            fTokenFactory = new TokenFactory();
-            fTokenRegistry = new RefHashTableOf<RangeTokenElemMap>(109);
-            fRangeMap = new RefHashTableOf<RangeFactory>(29);
-            fCategories = new XMLStringPool(109);
-            fRegistryInitialized = true;
-        }
-    }
+    // Add xml range factory
+    RangeFactory* rangeFact = new XMLRangeFactory();
+    fRangeMap->put((void*)fgXMLCategory, rangeFact);
+    rangeFact->initializeKeywordMap(this);
+
+    // Add ascii range factory
+    rangeFact = new ASCIIRangeFactory();
+    fRangeMap->put((void*)fgASCIICategory, rangeFact);
+    rangeFact->initializeKeywordMap(this);
+
+    // Add unicode range factory
+    rangeFact = new UnicodeRangeFactory();
+    fRangeMap->put((void*)fgUnicodeCategory, rangeFact);
+    rangeFact->initializeKeywordMap(this);
+
+    // Add block range factory
+    rangeFact = new BlockRangeFactory();
+    fRangeMap->put((void*)fgBlockCategory, rangeFact);
+    rangeFact->initializeKeywordMap(this);
 }
 
+void RangeTokenMap::buildTokenRanges()
+{
+    // Build ranges */
+    RangeFactory* rangeFactory = fRangeMap->get(fgXMLCategory);
+    rangeFactory->buildRanges(this);
+
+    rangeFactory = fRangeMap->get(fgASCIICategory);
+    rangeFactory->buildRanges(this);
+
+    rangeFactory = fRangeMap->get(fgUnicodeCategory);
+    rangeFactory->buildRanges(this);
+
+    rangeFactory = fRangeMap->get(fgBlockCategory);
+    rangeFactory->buildRanges(this);
+}
 
 // ---------------------------------------------------------------------------
 //  RangeTokenMap: Instance methods
@@ -314,7 +298,7 @@ RangeTokenMap* RangeTokenMap::instance()
 
         if (!fInstance)
         {
-            fInstance = new RangeTokenMap();
+            fInstance = new RangeTokenMap(XMLPlatformUtils::fgMemoryManager);
             rangeTokMapInstanceCleanup.registerCleanup(RangeTokenMap::reinitInstance);
         }
     }
@@ -322,6 +306,24 @@ RangeTokenMap* RangeTokenMap::instance()
     return (fInstance);
 }
 
+// ---------------------------------------------------------------------------
+//  RangeTokenMap: helper methods
+// ---------------------------------------------------------------------------
+void RangeTokenMap::cleanUp()
+{
+    delete fTokenRegistry;
+    fTokenRegistry = 0;
+
+    delete fRangeMap;
+    fRangeMap = 0;
+
+    delete fCategories;
+    fCategories = 0;
+
+    delete fTokenFactory;
+    fTokenFactory = 0;
+}
+
 // -----------------------------------------------------------------------
 //  Notification that lazy data has been deleted
 // -----------------------------------------------------------------------
@@ -329,7 +331,6 @@ void RangeTokenMap::reinitInstance() {
 
     delete fInstance;
     fInstance = 0;
-    TokenFactory::fRangeInitialized = false;
 }
 
 XERCES_CPP_NAMESPACE_END
