@@ -1,61 +1,22 @@
 /*
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2001 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache\@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation, and was
- * originally based on software copyright (c) 2001, International
- * Business Machines, Inc., http://www.ibm.com .  For more information
- * on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 /*
- * $Id: InitTermTest.cpp,v 1.3 2002/02/05 13:26:09 tng Exp $
+ * $Id: InitTermTest.cpp 568078 2007-08-21 11:43:25Z amassari $
  */
 
 // ---------------------------------------------------------------------------
@@ -70,19 +31,22 @@
 #include <xercesc/sax/SAXException.hpp>
 #include <xercesc/sax/SAXParseException.hpp>
 
-#include <xercesc/parsers/DOMParser.hpp>
-#include <xercesc/parsers/IDOMParser.hpp>
+#include <xercesc/parsers/XercesDOMParser.hpp>
 #include <xercesc/parsers/SAXParser.hpp>
 #include <xercesc/sax2/SAX2XMLReader.hpp>
 #include <xercesc/sax2/XMLReaderFactory.hpp>
-
-#include <xercesc/dom/DOM_DOMException.hpp>
-#include <xercesc/idom/IDOM_DOMException.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
+#include <xercesc/dom/DOMException.hpp>
 #include "InitTermTest.hpp"
 
 #include <string.h>
 #include <stdlib.h>
+
+#if defined(XERCES_NEW_IOSTREAMS)
+#include <fstream>
+#else
 #include <fstream.h>
+#endif
 #include <limits.h>
 
 
@@ -101,7 +65,6 @@ enum Teststate {
 //  Declare functions
 // ---------------------------------------------------------------------------
 int TestInit4DOM(const char* xmlFile, bool gDoNamespaces, bool gDoSchema, bool gSchemaFullChecking, Teststate theState);
-int TestInit4DOM(const char* xmlFile, bool gDoNamespaces, bool gDoSchema, bool gSchemaFullChecking, Teststate theState);
 int TestInit4SAX(const char* xmlFile, bool gDoNamespaces, bool gDoSchema, bool gSchemaFullChecking, Teststate theState);
 int TestInit4SAX2(const char* xmlFile, bool gDoNamespaces, bool gDoSchema, bool gSchemaFullChecking, Teststate theState);
 
@@ -132,8 +95,8 @@ int TestInit4SAX2(const char* xmlFile, bool gDoNamespaces, bool gDoSchema, bool
                                                                   \
         catch (const XMLException& toCatch)                       \
         {                                                         \
-             cerr << "Error during initialization! :\n"           \
-                  << StrX(toCatch.getMessage()) << endl;          \
+             XERCES_STD_QUALIFIER cerr << "Error during initialization! :\n"           \
+                  << StrX(toCatch.getMessage()) << XERCES_STD_QUALIFIER endl;          \
              return 1;                                            \
         }                                                         \
     }                                                             \
@@ -146,8 +109,8 @@ int TestInit4SAX2(const char* xmlFile, bool gDoNamespaces, bool gDoSchema, bool
                                                                   \
         catch (const XMLException& toCatch)                       \
         {                                                         \
-             cerr << "Error during initialization! :\n"           \
-                  << StrX(toCatch.getMessage()) << endl;          \
+             XERCES_STD_QUALIFIER cerr << "Error during initialization! :\n"           \
+                  << StrX(toCatch.getMessage()) << XERCES_STD_QUALIFIER endl;          \
              return 1;                                            \
         }                                                         \
     }
@@ -164,31 +127,35 @@ int TestInit4SAX2(const char* xmlFile, bool gDoNamespaces, bool gDoSchema, bool
     {                                                             \
         parser->parse(xmlFile);                                   \
     }                                                             \
-                                                                  \
-    catch (const XMLException& toCatch)                           \
+    catch (const OutOfMemoryException&)                           \
     {                                                             \
-        cerr << "\nError during parsing: '" << xmlFile << "'\n"   \
-             << "Exception message is:  \n"                       \
-             << StrX(toCatch.getMessage()) << "\n" << endl;       \
+        XERCES_STD_QUALIFIER cerr << "OutOfMemoryException" << XERCES_STD_QUALIFIER endl; \
         errorOccurred = true;                                     \
     }                                                             \
-    catch (const DOM_DOMException& toCatch)                       \
+    catch (const XMLException& toCatch)                           \
     {                                                             \
-        cerr << "\nDOM Error during parsing: '" << xmlFile        \
+        XERCES_STD_QUALIFIER cerr << "\nError during parsing: '" << xmlFile << "'\n"   \
+             << "Exception message is:  \n"                       \
+             << StrX(toCatch.getMessage()) << "\n" << XERCES_STD_QUALIFIER endl;       \
+        errorOccurred = true;                                     \
+    }                                                             \
+    catch (const DOMException& toCatch)                           \
+    {                                                             \
+        XERCES_STD_QUALIFIER cerr << "\nDOM Error during parsing: '" << xmlFile        \
              << "\nDOMException code is:  \n"                     \
-             << toCatch.code << "\n" << endl;                     \
+             << toCatch.code << "\n" << XERCES_STD_QUALIFIER endl;                     \
         errorOccurred = true;                                     \
     }                                                             \
     catch (...)                                                   \
     {                                                             \
-        cerr << "\nUnexpected exception during parsing: '"        \
+        XERCES_STD_QUALIFIER cerr << "\nUnexpected exception during parsing: '"        \
         << xmlFile << "'\n";                                      \
         errorOccurred = true;                                     \
     }                                                             \
                                                                   \
     if (errorHandler.getSawErrors())                              \
     {                                                             \
-        cout << "\nErrors occured, no output available\n" << endl;\
+        XERCES_STD_QUALIFIER cout << "\nErrors occurred, no output available\n" << XERCES_STD_QUALIFIER endl;\
         errorOccurred = true;                                     \
     }                                                             \
                                                                   \
@@ -213,20 +180,7 @@ int TestInit4SAX2(const char* xmlFile, bool gDoNamespaces, bool gDoSchema, bool
 int TestInit4DOM(const char* xmlFile, bool gDoNamespaces, bool gDoSchema, bool gSchemaFullChecking, Teststate theState)
 {
     TESTINITPRE;
-    DOMParser* parser = new DOMParser;
-    parser->setDoNamespaces(gDoNamespaces);
-    parser->setDoSchema(gDoSchema);
-    parser->setValidationSchemaFullChecking(gSchemaFullChecking);
-    TESTINITPOST;
-}
-
-// ---------------------------------------------------------------------------
-//  IDOM Parser
-// ---------------------------------------------------------------------------
-int TestInit4IDOM(const char* xmlFile, bool gDoNamespaces, bool gDoSchema, bool gSchemaFullChecking, Teststate theState)
-{
-    TESTINITPRE;
-    IDOMParser* parser = new IDOMParser;
+    XercesDOMParser* parser = new XercesDOMParser;
     parser->setDoNamespaces(gDoNamespaces);
     parser->setDoSchema(gDoSchema);
     parser->setValidationSchemaFullChecking(gSchemaFullChecking);
@@ -263,9 +217,9 @@ int TestInit4SAX2(const char* xmlFile, bool gDoNamespaces, bool gDoSchema, bool
     XMLCh* fullSchemaCheckFeature = XMLString::transcode("http://apache.org/xml/features/validation/schema-full-checking");
     parser->setFeature(fullSchemaCheckFeature, gSchemaFullChecking);
 
-    delete [] doNamespaceFeature;
-    delete [] doSchemaFeature;
-    delete [] fullSchemaCheckFeature;
+    XMLString::release(&doNamespaceFeature);
+    XMLString::release(&doSchemaFeature);
+    XMLString::release(&fullSchemaCheckFeature);
 
     TESTINITPOST;
 }
@@ -277,18 +231,16 @@ int TestInit4SAX2(const char* xmlFile, bool gDoNamespaces, bool gDoSchema, bool
 // ---------------------------------------------------------------------------
 void usage()
 {
-    cout << "\nUsage:\n"
+    XERCES_STD_QUALIFIER cout << "\nUsage:\n"
             "    InitTermTest [options] <XML file>\n\n"
             "This program tests the XMLPlatformUtils::Initialize()/Terminate()\n"
-            "pair by calling it a number of times.  All four parsers\n"
-            "(DOMParser, IDOMParser, SAXParser and SAX2XMLReader) are invoked\n"
-            "to parse the specified XML file.\n\n"
+            "pair by calling it a number of times.\n"
             "Options:\n"
             "    -n          Enable namespace processing. Default is off.\n"
             "    -s          Enable schema processing. Default is off.\n"
             "    -f          Enable full schema constraint checking. Defaults to off.\n"
 		      "    -?          Show this help.\n"
-          <<  endl;
+          <<  XERCES_STD_QUALIFIER endl;
 }
 
 // ---------------------------------------------------------------------------
@@ -351,8 +303,8 @@ int main(int argC, char* argV[]) {
         }
          else
         {
-            cerr << "Unknown option '" << argV[parmInd]
-                 << "', ignoring it.\n" << endl;
+            XERCES_STD_QUALIFIER cerr << "Unknown option '" << argV[parmInd]
+                 << "', ignoring it.\n" << XERCES_STD_QUALIFIER endl;
         }
     }
 
@@ -375,14 +327,12 @@ int main(int argC, char* argV[]) {
     unsigned int i = 0;
     for (i=0; i < 100; i++) {
         if (error) {
-            cout << "Test Failed" << endl;
+            XERCES_STD_QUALIFIER cout << "Test Failed" << XERCES_STD_QUALIFIER endl;
             return 4;
         }
 
         if (TestInit4DOM(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, Once))
             error = true;
-        if (TestInit4IDOM(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, Once))
-            error = true;
         if (TestInit4SAX(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, Once))
             error = true;
         if (TestInit4SAX2(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, Once))
@@ -391,8 +341,6 @@ int main(int argC, char* argV[]) {
 
     if (error || TestInit4DOM(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, Multiple))
         error = true;
-    if (error || TestInit4IDOM(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, Multiple))
-        error = true;
     if (error || TestInit4SAX(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, Multiple))
         error = true;
     if (error || TestInit4SAX2(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, Multiple))
@@ -404,8 +352,6 @@ int main(int argC, char* argV[]) {
 /*
     if (error || TestInit4DOM(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, Limit))
         error = true;
-    if (error || TestInit4IDOM(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, Limit))
-        error = true;
     if (error || TestInit4SAX(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, Limit))
         error = true;
     if (error || TestInit4SAX2(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, Limit))
@@ -413,8 +359,6 @@ int main(int argC, char* argV[]) {
 
     if (error || TestInit4DOM(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, ExceedLimit))
         error = true;
-    if (error || TestInit4IDOM(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, ExceedLimit))
-        error = true;
     if (error || TestInit4SAX(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, ExceedLimit))
         error = true;
     if (error || TestInit4SAX2(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, ExceedLimit))
@@ -423,19 +367,17 @@ int main(int argC, char* argV[]) {
 
     if (error || TestInit4DOM(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, UnEven))
         error = true;
-    if (error || TestInit4IDOM(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, UnEven))
-        error = true;
     if (error || TestInit4SAX(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, UnEven))
         error = true;
     if (error || TestInit4SAX2(xmlFile, gDoNamespaces, gDoSchema, gSchemaFullChecking, UnEven))
         error = true;
 
     if (error) {
-        cout << "Test Failed" << endl;
+        XERCES_STD_QUALIFIER cout << "Test Failed" << XERCES_STD_QUALIFIER endl;
         return 4;
     }
 
-    cout << "Test Run Successfully" << endl;
+    XERCES_STD_QUALIFIER cout << "Test Run Successfully" << XERCES_STD_QUALIFIER endl;
     return 0;
 }
 
@@ -455,27 +397,27 @@ InitTermTestErrorHandler::~InitTermTestErrorHandler()
 void InitTermTestErrorHandler::error(const SAXParseException& e)
 {
     fSawErrors = true;
-    cerr << "\nError at file " << StrX(e.getSystemId())
+    XERCES_STD_QUALIFIER cerr << "\nError at file " << StrX(e.getSystemId())
          << ", line " << e.getLineNumber()
          << ", char " << e.getColumnNumber()
-         << "\n  Message: " << StrX(e.getMessage()) << endl;
+         << "\n  Message: " << StrX(e.getMessage()) << XERCES_STD_QUALIFIER endl;
 }
 
 void InitTermTestErrorHandler::fatalError(const SAXParseException& e)
 {
     fSawErrors = true;
-    cerr << "\nFatal Error at file " << StrX(e.getSystemId())
+    XERCES_STD_QUALIFIER cerr << "\nFatal Error at file " << StrX(e.getSystemId())
          << ", line " << e.getLineNumber()
          << ", char " << e.getColumnNumber()
-         << "\n  Message: " << StrX(e.getMessage()) << endl;
+         << "\n  Message: " << StrX(e.getMessage()) << XERCES_STD_QUALIFIER endl;
 }
 
 void InitTermTestErrorHandler::warning(const SAXParseException& e)
 {
-    cerr << "\nWarning at file " << StrX(e.getSystemId())
+    XERCES_STD_QUALIFIER cerr << "\nWarning at file " << StrX(e.getSystemId())
          << ", line " << e.getLineNumber()
          << ", char " << e.getColumnNumber()
-         << "\n  Message: " << StrX(e.getMessage()) << endl;
+         << "\n  Message: " << StrX(e.getMessage()) << XERCES_STD_QUALIFIER endl;
 }
 
 void InitTermTestErrorHandler::resetErrors()
